@@ -14,7 +14,7 @@ See the License for the specific language governing permissions and
 limitations under the License.
 */
 
-package activator
+package net
 
 import (
 	"errors"
@@ -29,8 +29,6 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/sets"
-	kubeinformers "k8s.io/client-go/informers"
-	kubefake "k8s.io/client-go/kubernetes/fake"
 
 	fakekubeclient "knative.dev/pkg/client/injection/kube/client/fake"
 	fakeendpointsinformer "knative.dev/pkg/client/injection/kube/informers/core/v1/endpoints/fake"
@@ -52,12 +50,16 @@ import (
 )
 
 const (
-	testNamespace      = "test-namespace"
-	testRevision       = "test-revision"
-	informerRestPeriod = 2 * time.Second
+	testNamespace = "test-namespace"
+	testRevision  = "test-revision"
 )
 
-func revision(revID types.NamespacedName, protocol networking.ProtocolType) *v1alpha1.Revision {
+// revisionCC1 - creates a revision with concurrency == 1.
+func revisionCC1(revID types.NamespacedName, protocol networking.ProtocolType) *v1alpha1.Revision {
+	return revision(revID, protocol, 1)
+}
+
+func revision(revID types.NamespacedName, protocol networking.ProtocolType, cc int64) *v1alpha1.Revision {
 	return &v1alpha1.Revision{
 		ObjectMeta: metav1.ObjectMeta{
 			Namespace: revID.Namespace,
@@ -65,7 +67,7 @@ func revision(revID types.NamespacedName, protocol networking.ProtocolType) *v1a
 		},
 		Spec: v1alpha1.RevisionSpec{
 			RevisionSpec: v1.RevisionSpec{
-				ContainerConcurrency: ptr.Int64(1),
+				ContainerConcurrency: ptr.Int64(cc),
 				PodSpec: corev1.PodSpec{
 					Containers: []corev1.Container{{
 						Ports: []corev1.ContainerPort{{
@@ -78,7 +80,7 @@ func revision(revID types.NamespacedName, protocol networking.ProtocolType) *v1a
 	}
 }
 
-func privateSksService(revID types.NamespacedName, clusterIP string, ports []corev1.ServicePort) *corev1.Service {
+func privateSKSService(revID types.NamespacedName, clusterIP string, ports []corev1.ServicePort) *corev1.Service {
 	return &corev1.Service{
 		ObjectMeta: metav1.ObjectMeta{
 			Namespace: revID.Namespace,
@@ -95,17 +97,34 @@ func privateSksService(revID types.NamespacedName, clusterIP string, ports []cor
 	}
 }
 
+func waitForRevisionBackedMananger(t *testing.T, rbm *revisionBackendsManager) {
+	timeout := time.After(200 * time.Millisecond)
+	for {
+		select {
+		// rbm.updates() gets closed after all revisionWatchers have finished
+		case _, ok := <-rbm.updates():
+			if !ok {
+				return
+			}
+		case <-timeout:
+			t.Error("Timed out waiting for revisionBackendManager to finish")
+			return
+		}
+	}
+}
+
 func TestRevisionWatcher(t *testing.T) {
+	logger := TestLogger(t)
 	for _, tc := range []struct {
 		name                  string
 		dests                 []string
 		protocol              networking.ProtocolType
 		clusterPort           corev1.ServicePort
 		clusterIP             string
-		expectUpdates         []RevisionDestsUpdate
+		expectUpdates         []revisionDestsUpdate
 		probeHostResponses    map[string][]activatortest.FakeResponse
-		probeResponses        []activatortest.FakeResponse
 		initialClusterIPState bool
+		noPodAddressability   bool // This keeps the test defs shorter.
 	}{{
 		name:  "single healthy podIP",
 		dests: []string{"128.0.0.1:1234"},
@@ -114,13 +133,13 @@ func TestRevisionWatcher(t *testing.T) {
 			Port: 1234,
 		},
 		clusterIP:     "129.0.0.1",
-		expectUpdates: []RevisionDestsUpdate{{Dests: sets.NewString("128.0.0.1:1234")}},
-		probeResponses: []activatortest.FakeResponse{{
-			Err: errors.New("clusterIP transport error"),
-		}, {
-			Code: http.StatusOK,
-			Body: queue.Name,
-		}},
+		expectUpdates: []revisionDestsUpdate{{Dests: sets.NewString("128.0.0.1:1234")}},
+		probeHostResponses: map[string][]activatortest.FakeResponse{
+			"128.0.0.1:1234": {{
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}},
+		},
 	}, {
 		name:     "single http2 podIP",
 		dests:    []string{"128.0.0.1:1234"},
@@ -130,7 +149,7 @@ func TestRevisionWatcher(t *testing.T) {
 			Port: 1234,
 		},
 		clusterIP:     "129.0.0.1",
-		expectUpdates: []RevisionDestsUpdate{{Dests: sets.NewString("128.0.0.1:1234")}},
+		expectUpdates: []revisionDestsUpdate{{Dests: sets.NewString("128.0.0.1:1234")}},
 		probeHostResponses: map[string][]activatortest.FakeResponse{
 			"129.0.0.1:1234": {{
 				Err: errors.New("clusterIP transport error"),
@@ -148,8 +167,9 @@ func TestRevisionWatcher(t *testing.T) {
 			Name: "http2",
 			Port: 1234,
 		},
-		clusterIP:     "129.0.0.1",
-		expectUpdates: []RevisionDestsUpdate{{ClusterIPDest: "129.0.0.1:1234", Dests: sets.NewString("128.0.0.1:1234")}},
+		clusterIP:           "129.0.0.1",
+		noPodAddressability: true,
+		expectUpdates:       []revisionDestsUpdate{{ClusterIPDest: "129.0.0.1:1234", Dests: sets.NewString("128.0.0.1:1234")}},
 		probeHostResponses: map[string][]activatortest.FakeResponse{
 			"129.0.0.1:1234": {{
 				Code: http.StatusOK,
@@ -176,10 +196,14 @@ func TestRevisionWatcher(t *testing.T) {
 			Port: 1234,
 		},
 		clusterIP: "129.0.0.1",
-		probeResponses: []activatortest.FakeResponse{{
-			Code: http.StatusServiceUnavailable,
-			Body: queue.Name,
-		}},
+		probeHostResponses: map[string][]activatortest.FakeResponse{
+			"129.0.0.1:1234": {{
+				Code: http.StatusServiceUnavailable,
+			}},
+			"128.0.0.1:1234": {{
+				Code: http.StatusServiceUnavailable,
+			}},
+		},
 	}, {
 		name:  "single error podIP",
 		dests: []string{"128.0.0.1:1234"},
@@ -188,11 +212,14 @@ func TestRevisionWatcher(t *testing.T) {
 			Port: 1234,
 		},
 		clusterIP: "129.0.0.1",
-		probeResponses: []activatortest.FakeResponse{{
-			Err:  errors.New("Fake error"),
-			Code: http.StatusOK,
-			Body: queue.Name,
-		}},
+		probeHostResponses: map[string][]activatortest.FakeResponse{
+			"129.0.0.1:1234": {{
+				Code: http.StatusServiceUnavailable,
+			}},
+			"128.0.0.1:1234": {{
+				Err: errors.New("Fake error"),
+			}},
+		},
 	}, {
 		name:  "podIP slow ready",
 		dests: []string{"128.0.0.1:1234"},
@@ -201,18 +228,18 @@ func TestRevisionWatcher(t *testing.T) {
 			Port: 1234,
 		},
 		clusterIP:     "129.0.0.1",
-		expectUpdates: []RevisionDestsUpdate{{Dests: sets.NewString("128.0.0.1:1234")}},
-		probeResponses: []activatortest.FakeResponse{{
-			Err: errors.New("clusterIP transport error"),
-		}, {
-			Code: http.StatusServiceUnavailable,
-			Body: queue.Name,
-		}, {
-			Err: errors.New("clusterIP transport error"),
-		}, {
-			Code: http.StatusOK,
-			Body: queue.Name,
-		}},
+		expectUpdates: []revisionDestsUpdate{{Dests: sets.NewString("128.0.0.1:1234")}},
+		probeHostResponses: map[string][]activatortest.FakeResponse{
+			"129.0.0.1:1234": {{
+				Err: errors.New("clusterIP transport error"),
+			}},
+			"128.0.0.1:1234": {{
+				Err: errors.New("clusterIP transport error"),
+			}, {
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}},
+		},
 	}, {
 		name:  "multiple healthy podIP",
 		dests: []string{"128.0.0.1:1234", "128.0.0.2:1234"},
@@ -221,13 +248,17 @@ func TestRevisionWatcher(t *testing.T) {
 			Port: 1234,
 		},
 		clusterIP:     "129.0.0.1",
-		expectUpdates: []RevisionDestsUpdate{{Dests: sets.NewString("128.0.0.1:1234", "128.0.0.2:1234")}},
-		probeResponses: []activatortest.FakeResponse{{
-			Err: errors.New("clusterIP transport error"),
-		}, {
-			Code: http.StatusOK,
-			Body: queue.Name,
-		}},
+		expectUpdates: []revisionDestsUpdate{{Dests: sets.NewString("128.0.0.1:1234", "128.0.0.2:1234")}},
+		probeHostResponses: map[string][]activatortest.FakeResponse{
+			"128.0.0.1:1234": {{
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}},
+			"128.0.0.2:1234": {{
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}},
+		},
 	}, {
 		name:  "one healthy one unhealthy podIP",
 		dests: []string{"128.0.0.1:1234", "128.0.0.2:1234"},
@@ -236,11 +267,8 @@ func TestRevisionWatcher(t *testing.T) {
 			Port: 1234,
 		},
 		clusterIP:     "129.0.0.1",
-		expectUpdates: []RevisionDestsUpdate{{Dests: sets.NewString("128.0.0.2:1234")}},
+		expectUpdates: []revisionDestsUpdate{{Dests: sets.NewString("128.0.0.2:1234")}},
 		probeHostResponses: map[string][]activatortest.FakeResponse{
-			"129.0.0.1:1234": {{
-				Err: errors.New("clusterIP transport error"),
-			}},
 			"128.0.0.1:1234": {{
 				Err: errors.New("clusterIP transport error"),
 			}},
@@ -250,22 +278,47 @@ func TestRevisionWatcher(t *testing.T) {
 			}},
 		},
 	}, {
-		name:  "one healthy one unhealthy podIP then 2, then cluster",
+		name:  "one healthy one unhealthy podIP then both healthy",
 		dests: []string{"128.0.0.1:1234", "128.0.0.2:1234"},
 		clusterPort: corev1.ServicePort{
 			Name: "http",
 			Port: 4321,
 		},
 		clusterIP: "129.0.0.1",
-		expectUpdates: []RevisionDestsUpdate{
+		expectUpdates: []revisionDestsUpdate{
 			{Dests: sets.NewString("128.0.0.2:1234")},
 			{Dests: sets.NewString("128.0.0.2:1234", "128.0.0.1:1234")},
-			{ClusterIPDest: "129.0.0.1:4321", Dests: sets.NewString("128.0.0.2:1234", "128.0.0.1:1234")},
 		},
 		probeHostResponses: map[string][]activatortest.FakeResponse{
-			"129.0.0.1:4321": {{
+			"128.0.0.1:1234": {{
 				Err: errors.New("clusterIP transport error"),
 			}, {
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}},
+			"128.0.0.2:1234": {{
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}, {
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}},
+		},
+	}, {
+		name:  "clusterIP slow ready, no pod addressability",
+		dests: []string{"128.0.0.1:1234"},
+		clusterPort: corev1.ServicePort{
+			Name: "http",
+			Port: 1234,
+		},
+		clusterIP: "129.0.0.1",
+		expectUpdates: []revisionDestsUpdate{{
+			ClusterIPDest: "129.0.0.1:1234",
+			Dests:         sets.NewString("128.0.0.1:1234"),
+		}},
+		noPodAddressability: true,
+		probeHostResponses: map[string][]activatortest.FakeResponse{
+			"129.0.0.1:1234": {{
 				Err: errors.New("clusterIP transport error"),
 			}, {
 				Code: http.StatusOK,
@@ -274,59 +327,45 @@ func TestRevisionWatcher(t *testing.T) {
 			"128.0.0.1:1234": {{
 				Err: errors.New("podIP transport error"),
 			}, {
-				Code: http.StatusOK,
-				Body: queue.Name,
-			}},
-			"128.0.0.2:1234": {{
-				Code: http.StatusOK,
-				Body: queue.Name,
+				Err: errors.New("podIP transport error"),
 			}},
 		},
 	}, {
-		name:  "podIP slow ready then clusterIP",
+		name:  "clusterIP  ready, no pod addressability",
 		dests: []string{"128.0.0.1:1234"},
 		clusterPort: corev1.ServicePort{
 			Name: "http",
-			Port: 1234,
-		},
-		clusterIP: "129.0.0.1",
-		expectUpdates: []RevisionDestsUpdate{
-			{Dests: sets.NewString("128.0.0.1:1234")},
-			{Dests: sets.NewString("128.0.0.1:1234"), ClusterIPDest: "129.0.0.1:1234"},
+			Port: 1235,
 		},
+		noPodAddressability: true,
+		clusterIP:           "129.0.0.1",
+		expectUpdates: []revisionDestsUpdate{{
+			ClusterIPDest: "129.0.0.1:1235",
+			Dests:         sets.NewString("128.0.0.1:1234"),
+		}},
 		probeHostResponses: map[string][]activatortest.FakeResponse{
 			"129.0.0.1:1234": {{
-				Err: errors.New("clusterIP transport error"),
-			}, {
-				Err: errors.New("clusterIP transport error"),
-			}, {
 				Code: http.StatusOK,
 				Body: queue.Name,
 			}},
 			"128.0.0.1:1234": {{
-				Code: http.StatusServiceUnavailable,
-				Body: queue.Name,
-			}, {
-				Code: http.StatusOK,
-				Body: queue.Name,
+				Err: errors.New("podIP transport error"),
 			}},
 		},
 	}} {
 		t.Run(tc.name, func(t *testing.T) {
-			defer ClearAll()
 			fakeRT := activatortest.FakeRoundTripper{
 				ExpectHost:         testRevision,
 				ProbeHostResponses: tc.probeHostResponses,
-				ProbeResponses:     tc.probeResponses,
 			}
 			rt := network.RoundTripperFunc(fakeRT.RT)
 
-			doneCh := make(chan struct{})
-			defer close(doneCh)
+			ctx, cancel, _ := rtesting.SetupFakeContextWithCancel(t)
 
-			updateCh := make(chan RevisionDestsUpdate, len(tc.expectUpdates)+1)
+			updateCh := make(chan revisionDestsUpdate, len(tc.expectUpdates)+1)
+			defer close(updateCh)
 
-			// This gets cleaned up as part of the test
+			// This gets closed up by revisionWatcher
 			destsCh := make(chan sets.String)
 
 			// Default for protocol is http1
@@ -334,26 +373,34 @@ func TestRevisionWatcher(t *testing.T) {
 				tc.protocol = networking.ProtocolHTTP1
 			}
 
-			fake := kubefake.NewSimpleClientset()
-			informer := kubeinformers.NewSharedInformerFactory(fake, 0)
-			servicesLister := informer.Core().V1().Services().Lister()
+			fake := fakekubeclient.Get(ctx)
+			informer := fakeserviceinformer.Get(ctx)
 
 			revID := types.NamespacedName{Namespace: testNamespace, Name: testRevision}
 			if tc.clusterIP != "" {
-				svc := privateSksService(revID, tc.clusterIP, []corev1.ServicePort{tc.clusterPort})
+				svc := privateSKSService(revID, tc.clusterIP, []corev1.ServicePort{tc.clusterPort})
 				fake.CoreV1().Services(svc.Namespace).Create(svc)
-				informer.Core().V1().Services().Informer().GetIndexer().Add(svc)
+				informer.Informer().GetIndexer().Add(svc)
 			}
 
+			waitInformers, err := controller.RunInformers(ctx.Done(), informer.Informer())
+			if err != nil {
+				t.Fatalf("Failed to start informers: %v", err)
+			}
+			defer func() {
+				cancel()
+				waitInformers()
+			}()
+
 			rw := newRevisionWatcher(
-				doneCh,
+				ctx,
 				revID,
 				tc.protocol,
 				updateCh,
 				destsCh,
 				rt,
-				servicesLister,
-				TestLogger(t),
+				informer.Lister(),
+				logger,
 			)
 			rw.clusterIPHealthy = tc.initialClusterIPState
 
@@ -366,7 +413,7 @@ func TestRevisionWatcher(t *testing.T) {
 
 			destsCh <- sets.NewString(tc.dests...)
 
-			updates := []RevisionDestsUpdate{}
+			updates := []revisionDestsUpdate{}
 			for i := 0; i < len(tc.expectUpdates); i++ {
 				select {
 				case update := <-updateCh:
@@ -375,11 +422,15 @@ func TestRevisionWatcher(t *testing.T) {
 					t.Error("Timed out waiting for update event")
 				}
 			}
+			if got, want := rw.podsAddressable, !tc.noPodAddressability; got != want {
+				t.Errorf("Revision pod addressability = %v, want: %v", got, want)
+			}
 
 			// Shutdown run loop.
-			close(destsCh)
+			cancel()
 
 			wg.Wait()
+			assertChClosed(t, rw.done)
 
 			// Autofill out Rev in expectUpdates
 			for i := range tc.expectUpdates {
@@ -393,6 +444,20 @@ func TestRevisionWatcher(t *testing.T) {
 	}
 }
 
+func assertChClosed(t *testing.T, ch chan struct{}) {
+	defer func() {
+		if r := recover(); r == nil {
+			t.Errorf("the channel was not closed")
+		}
+	}()
+	select {
+	case ch <- struct{}{}:
+		// Panics if the channel is closed
+	default:
+		// Prevents from blocking forever if the channel is not closed
+	}
+}
+
 func epSubset(port int32, portName string, ips []string) *corev1.EndpointSubset {
 	ss := &corev1.EndpointSubset{
 		Ports: []corev1.EndpointPort{{
@@ -421,27 +486,23 @@ func ep(revL string, port int32, portName string, ips ...string) *corev1.Endpoin
 }
 
 func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
-	logger := TestLogger(t)
-	defer ClearAll()
 	// Make sure we wait out all the jitter in the system.
-	defer time.Sleep(informerRestPeriod)
 	for _, tc := range []struct {
 		name               string
 		endpointsArr       []*corev1.Endpoints
 		revisions          []*v1alpha1.Revision
 		services           []*corev1.Service
-		probeResponses     []activatortest.FakeResponse
 		probeHostResponses map[string][]activatortest.FakeResponse
-		expectDests        map[types.NamespacedName]RevisionDestsUpdate
+		expectDests        map[types.NamespacedName]revisionDestsUpdate
 		updateCnt          int
 	}{{
 		name:         "add slow healthy",
 		endpointsArr: []*corev1.Endpoints{ep(testRevision, 1234, "http", "128.0.0.1")},
 		revisions: []*v1alpha1.Revision{
-			revision(types.NamespacedName{testNamespace, testRevision}, networking.ProtocolHTTP1),
+			revisionCC1(types.NamespacedName{testNamespace, testRevision}, networking.ProtocolHTTP1),
 		},
 		services: []*corev1.Service{
-			privateSksService(types.NamespacedName{testNamespace, testRevision}, "129.0.0.1",
+			privateSKSService(types.NamespacedName{testNamespace, testRevision}, "129.0.0.1",
 				[]corev1.ServicePort{{Name: "http", Port: 1234}}),
 		},
 		probeHostResponses: map[string][]activatortest.FakeResponse{
@@ -456,7 +517,7 @@ func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
 				Body: queue.Name,
 			}},
 		},
-		expectDests: map[types.NamespacedName]RevisionDestsUpdate{
+		expectDests: map[types.NamespacedName]revisionDestsUpdate{
 			{Namespace: testNamespace, Name: testRevision}: {
 				Dests: sets.NewString("128.0.0.1:1234"),
 			},
@@ -466,10 +527,10 @@ func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
 		name:         "add slow ready http2",
 		endpointsArr: []*corev1.Endpoints{ep(testRevision, 1234, "http2", "128.0.0.1")},
 		revisions: []*v1alpha1.Revision{
-			revision(types.NamespacedName{testNamespace, testRevision}, networking.ProtocolH2C),
+			revisionCC1(types.NamespacedName{testNamespace, testRevision}, networking.ProtocolH2C),
 		},
 		services: []*corev1.Service{
-			privateSksService(types.NamespacedName{testNamespace, testRevision}, "129.0.0.1",
+			privateSKSService(types.NamespacedName{testNamespace, testRevision}, "129.0.0.1",
 				[]corev1.ServicePort{{Name: "http2", Port: 1234}}),
 		},
 		probeHostResponses: map[string][]activatortest.FakeResponse{
@@ -484,7 +545,7 @@ func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
 				Body: queue.Name,
 			}},
 		},
-		expectDests: map[types.NamespacedName]RevisionDestsUpdate{
+		expectDests: map[types.NamespacedName]revisionDestsUpdate{
 			{Namespace: testNamespace, Name: testRevision}: {
 				Dests: sets.NewString("128.0.0.1:1234"),
 			},
@@ -497,20 +558,20 @@ func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
 			ep("test-revision2", 1235, "http", "128.1.0.2"),
 		},
 		revisions: []*v1alpha1.Revision{
-			revision(types.NamespacedName{testNamespace, "test-revision1"}, networking.ProtocolHTTP1),
-			revision(types.NamespacedName{testNamespace, "test-revision2"}, networking.ProtocolHTTP1),
+			revisionCC1(types.NamespacedName{testNamespace, "test-revision1"}, networking.ProtocolHTTP1),
+			revisionCC1(types.NamespacedName{testNamespace, "test-revision2"}, networking.ProtocolHTTP1),
 		},
 		services: []*corev1.Service{
-			privateSksService(types.NamespacedName{testNamespace, "test-revision1"}, "129.0.0.1",
+			privateSKSService(types.NamespacedName{testNamespace, "test-revision1"}, "129.0.0.1",
 				[]corev1.ServicePort{{Name: "http", Port: 2345}}),
-			privateSksService(types.NamespacedName{testNamespace, "test-revision2"}, "129.0.0.2",
+			privateSKSService(types.NamespacedName{testNamespace, "test-revision2"}, "129.0.0.2",
 				[]corev1.ServicePort{{Name: "http", Port: 2345}}),
 		},
 		probeHostResponses: map[string][]activatortest.FakeResponse{
 			"129.0.0.1:2345": {{Err: errors.New("clusterIP transport error")}},
 			"129.0.0.2:2345": {{Err: errors.New("clusterIP transport error")}},
 		},
-		expectDests: map[types.NamespacedName]RevisionDestsUpdate{
+		expectDests: map[types.NamespacedName]revisionDestsUpdate{
 			{Namespace: testNamespace, Name: "test-revision1"}: {
 				Dests: sets.NewString("128.0.0.1:1234"),
 			},
@@ -520,47 +581,66 @@ func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
 		},
 		updateCnt: 2,
 	}, {
-		name:         "slow podIP then clusterIP",
+		name:         "slow podIP",
 		endpointsArr: []*corev1.Endpoints{ep(testRevision, 1234, "http", "128.0.0.1")},
 		revisions: []*v1alpha1.Revision{
-			revision(types.NamespacedName{testNamespace, testRevision}, networking.ProtocolHTTP1),
+			revisionCC1(types.NamespacedName{testNamespace, testRevision}, networking.ProtocolHTTP1),
 		},
 		services: []*corev1.Service{
-			privateSksService(types.NamespacedName{testNamespace, testRevision}, "129.0.0.1",
+			privateSKSService(types.NamespacedName{testNamespace, testRevision}, "129.0.0.1",
 				[]corev1.ServicePort{{Name: "http", Port: 1234}}),
 		},
 		probeHostResponses: map[string][]activatortest.FakeResponse{
 			"129.0.0.1:1234": {{
 				Err: errors.New("clusterIP transport error"),
-			}, {
-				Err: errors.New("clusterIP transport error"),
-			}, {
-				Code: http.StatusOK,
-				Body: queue.Name,
 			}},
 			"128.0.0.1:1234": {{
 				Code: http.StatusServiceUnavailable,
-				Body: queue.Name,
 			}, {
 				Code: http.StatusOK,
 				Body: queue.Name,
 			}},
 		},
-		expectDests: map[types.NamespacedName]RevisionDestsUpdate{
+		expectDests: map[types.NamespacedName]revisionDestsUpdate{
+			{Namespace: testNamespace, Name: testRevision}: {
+				Dests: sets.NewString("128.0.0.1:1234"),
+			},
+		},
+		updateCnt: 1,
+	}, {
+		name:         "no pod addressability",
+		endpointsArr: []*corev1.Endpoints{ep(testRevision, 1234, "http", "128.0.0.1")},
+		revisions: []*v1alpha1.Revision{
+			revisionCC1(types.NamespacedName{testNamespace, testRevision}, networking.ProtocolHTTP1),
+		},
+		services: []*corev1.Service{
+			privateSKSService(types.NamespacedName{testNamespace, testRevision}, "129.0.0.1",
+				[]corev1.ServicePort{{Name: "http", Port: 1234}}),
+		},
+		probeHostResponses: map[string][]activatortest.FakeResponse{
+			"129.0.0.1:1234": {{
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}},
+			"128.0.0.1:1234": {{
+				Code: http.StatusServiceUnavailable,
+			}},
+		},
+		expectDests: map[types.NamespacedName]revisionDestsUpdate{
 			{Namespace: testNamespace, Name: testRevision}: {
 				ClusterIPDest: "129.0.0.1:1234",
 				Dests:         sets.NewString("128.0.0.1:1234"),
 			},
 		},
-		updateCnt: 2,
+		updateCnt: 1,
 	}, {
 		name:         "unhealthy",
 		endpointsArr: []*corev1.Endpoints{ep(testRevision, 1234, "http", "128.0.0.1")},
 		revisions: []*v1alpha1.Revision{
-			revision(types.NamespacedName{testNamespace, testRevision}, networking.ProtocolHTTP1),
+			revisionCC1(types.NamespacedName{testNamespace, testRevision}, networking.ProtocolHTTP1),
 		},
 		services: []*corev1.Service{
-			privateSksService(types.NamespacedName{testNamespace, testRevision}, "129.0.0.1",
+			privateSKSService(types.NamespacedName{testNamespace, testRevision}, "129.0.0.1",
 				[]corev1.ServicePort{{Name: "http", Port: 1234}}),
 		},
 		probeHostResponses: map[string][]activatortest.FakeResponse{
@@ -572,22 +652,19 @@ func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
 				Body: queue.Name,
 			}},
 		},
-		expectDests: map[types.NamespacedName]RevisionDestsUpdate{},
+		expectDests: map[types.NamespacedName]revisionDestsUpdate{},
 	}} {
 		t.Run(tc.name, func(t *testing.T) {
 			fakeRT := activatortest.FakeRoundTripper{
 				ExpectHost:         testRevision,
 				ProbeHostResponses: tc.probeHostResponses,
-				ProbeResponses:     tc.probeResponses,
 			}
 			rt := network.RoundTripperFunc(fakeRT.RT)
 
 			ctx, cancel, _ := rtesting.SetupFakeContextWithCancel(t)
-			defer cancel()
 
 			endpointsInformer := fakeendpointsinformer.Get(ctx)
 			serviceInformer := fakeserviceinformer.Get(ctx)
-
 			revisions := fakerevisioninformer.Get(ctx)
 
 			// Add the revision we're testing.
@@ -601,20 +678,27 @@ func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
 				serviceInformer.Informer().GetIndexer().Add(svc)
 			}
 
-			controller.StartInformers(ctx.Done(), endpointsInformer.Informer())
+			waitInformers, err := controller.RunInformers(ctx.Done(), endpointsInformer.Informer())
+			if err != nil {
+				t.Fatalf("Failed to start informers: %v", err)
+			}
+			defer func() {
+				cancel()
+				waitInformers()
+			}()
 
-			rbm := NewRevisionBackendsManagerWithProbeFrequency(ctx, rt, logger, 50*time.Millisecond)
+			rbm := newRevisionBackendsManagerWithProbeFrequency(ctx, rt, 50*time.Millisecond)
 
 			for _, ep := range tc.endpointsArr {
 				fakekubeclient.Get(ctx).CoreV1().Endpoints(testNamespace).Create(ep)
 				endpointsInformer.Informer().GetIndexer().Add(ep)
 			}
 
-			revDests := make(map[types.NamespacedName]RevisionDestsUpdate)
+			revDests := make(map[types.NamespacedName]revisionDestsUpdate)
 			// Wait for updateCb to be called
 			for i := 0; i < tc.updateCnt; i++ {
 				select {
-				case update := <-rbm.UpdateCh():
+				case update := <-rbm.updates():
 					revDests[update.Rev] = update
 				case <-time.After(300 * time.Millisecond):
 					t.Errorf("Timed out waiting for update event")
@@ -630,6 +714,9 @@ func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
 			if got, want := revDests, tc.expectDests; !cmp.Equal(got, want) {
 				t.Errorf("RevisionDests = %v, want: %v, diff(-want,+got):%s\n", got, want, cmp.Diff(want, got))
 			}
+
+			cancel()
+			waitForRevisionBackedMananger(t, rbm)
 		})
 	}
 }
@@ -638,15 +725,9 @@ func TestCheckDests(t *testing.T) {
 	// This test covers some edge cases in `checkDests` which are next to impossible to
 	// test via tests above.
 
-	// To make sure context switch happens and informers terminate.
 	ctx, cancel, _ := rtesting.SetupFakeContextWithCancel(t)
-	defer func() {
-		cancel()
-		time.Sleep(informerRestPeriod)
-		ClearAll()
-	}()
 
-	svc := privateSksService(
+	svc := privateSKSService(
 		types.NamespacedName{testNamespace, testRevision},
 		"129.0.0.1",
 		[]corev1.ServicePort{{Name: "http", Port: 1234}},
@@ -654,21 +735,32 @@ func TestCheckDests(t *testing.T) {
 	fakekubeclient.Get(ctx).CoreV1().Services(testNamespace).Create(svc)
 	si := fakeserviceinformer.Get(ctx)
 	si.Informer().GetIndexer().Add(svc)
+
+	waitInformers, err := controller.RunInformers(ctx.Done(), si.Informer())
+	if err != nil {
+		t.Fatalf("Failed to start informers: %v", err)
+	}
+	defer func() {
+		cancel()
+		waitInformers()
+	}()
+
 	// Make it buffered,so that we can make the test linear.
-	uCh := make(chan RevisionDestsUpdate, 1)
+	uCh := make(chan revisionDestsUpdate, 1)
 	dCh := make(chan struct{})
 	rw := &revisionWatcher{
 		clusterIPHealthy: true,
+		podsAddressable:  false,
 		rev:              types.NamespacedName{testNamespace, testRevision},
 		updateCh:         uCh,
 		serviceLister:    si.Lister(),
 		logger:           TestLogger(t),
-		doneCh:           dCh,
+		stopCh:           dCh,
 	}
 	rw.checkDests(sets.NewString("10.1.1.5"))
 	select {
 	case <-uCh:
-		// success.
+		// Success.
 	default:
 		t.Error("Expected update but it never went out.")
 	}
@@ -679,7 +771,7 @@ func TestCheckDests(t *testing.T) {
 	case <-uCh:
 		t.Error("Expected no update but got one")
 	default:
-		// success.
+		// Success.
 	}
 }
 
@@ -687,13 +779,8 @@ func TestCheckDestsSwinging(t *testing.T) {
 	// This test permits us to test the case when endpoints actually change
 	// underneath (e.g. pod crash/restart).
 	ctx, cancel, _ := rtesting.SetupFakeContextWithCancel(t)
-	defer func() {
-		cancel()
-		time.Sleep(informerRestPeriod)
-		ClearAll()
-	}()
 
-	svc := privateSksService(
+	svc := privateSKSService(
 		types.NamespacedName{testNamespace, testRevision},
 		"10.5.0.1",
 		[]corev1.ServicePort{{Name: "http", Port: 1234}},
@@ -703,6 +790,15 @@ func TestCheckDestsSwinging(t *testing.T) {
 	si := fakeserviceinformer.Get(ctx)
 	si.Informer().GetIndexer().Add(svc)
 
+	waitInformers, err := controller.RunInformers(ctx.Done(), si.Informer())
+	if err != nil {
+		t.Fatalf("Failed to start informers: %v", err)
+	}
+	defer func() {
+		cancel()
+		waitInformers()
+	}()
+
 	fakeRT := activatortest.FakeRoundTripper{
 		ExpectHost: testRevision,
 		ProbeHostResponses: map[string][]activatortest.FakeResponse{
@@ -714,9 +810,6 @@ func TestCheckDestsSwinging(t *testing.T) {
 				Err: errors.New("clusterIP transport error"),
 			}, {
 				Err: errors.New("clusterIP transport error"),
-			}, {
-				Code: http.StatusOK,
-				Body: queue.Name,
 			}},
 			"10.0.0.1:1234": {{
 				Err: errors.New("podIP transport error"),
@@ -733,23 +826,28 @@ func TestCheckDestsSwinging(t *testing.T) {
 				Code: http.StatusOK,
 				Body: queue.Name,
 			}},
+			"10.0.0.3:1234": {{
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}},
 		},
 	}
 
 	// Make it buffered,so that we can make the test linear.
-	uCh := make(chan RevisionDestsUpdate, 1)
+	uCh := make(chan revisionDestsUpdate, 1)
 	dCh := make(chan struct{})
 	rw := &revisionWatcher{
-		rev:           types.NamespacedName{testNamespace, testRevision},
-		updateCh:      uCh,
-		serviceLister: si.Lister(),
-		logger:        TestLogger(t),
-		doneCh:        dCh,
-		transport:     network.RoundTripperFunc(fakeRT.RT),
+		rev:             types.NamespacedName{testNamespace, testRevision},
+		updateCh:        uCh,
+		serviceLister:   si.Lister(),
+		logger:          TestLogger(t),
+		stopCh:          dCh,
+		podsAddressable: true,
+		transport:       network.RoundTripperFunc(fakeRT.RT),
 	}
 	// First not ready, second good, clusterIP: not ready.
 	rw.checkDests(sets.NewString("10.0.0.1:1234", "10.0.0.2:1234"))
-	want := RevisionDestsUpdate{
+	want := revisionDestsUpdate{
 		Rev:           types.NamespacedName{testNamespace, testRevision},
 		ClusterIPDest: "",
 		Dests:         sets.NewString("10.0.0.2:1234"),
@@ -797,11 +895,32 @@ func TestCheckDestsSwinging(t *testing.T) {
 		t.Error("Expected update but it never went out.")
 	}
 
-	// And finally the cluster IP is ready.
+	// Make sure we do not send out redundant updates.
 	rw.checkDests(sets.NewString("10.0.0.1:1234", "10.0.0.2:1234"))
-	want.ClusterIPDest = "10.5.0.1:1234"
 	select {
 	case got := <-uCh:
+		t.Errorf("Expected no update, but got %#v", got)
+	default:
+		// Success.
+	}
+
+	// Swing to a different pods.
+	rw.checkDests(sets.NewString("10.0.0.3:1234", "10.0.0.2:1234"))
+	select {
+	case got := <-uCh:
+		want.Dests = sets.NewString("10.0.0.2:1234", "10.0.0.3:1234")
+		if !cmp.Equal(got, want) {
+			t.Errorf("Update = %#v, want: %#v, diff: %s", got, want, cmp.Diff(want, got))
+		}
+	default:
+		t.Error("Expected update but it never went out.")
+	}
+
+	// Scale down by 1.
+	rw.checkDests(sets.NewString("10.0.0.2:1234"))
+	select {
+	case got := <-uCh:
+		want.Dests = sets.NewString("10.0.0.2:1234")
 		if !cmp.Equal(got, want) {
 			t.Errorf("Update = %#v, want: %#v, diff: %s", got, want, cmp.Diff(want, got))
 		}
@@ -812,13 +931,8 @@ func TestCheckDestsSwinging(t *testing.T) {
 
 func TestRevisionDeleted(t *testing.T) {
 	ctx, cancel, _ := rtesting.SetupFakeContextWithCancel(t)
-	defer func() {
-		cancel()
-		time.Sleep(informerRestPeriod)
-		ClearAll()
-	}()
 
-	svc := privateSksService(
+	svc := privateSKSService(
 		types.NamespacedName{testNamespace, testRevision},
 		"129.0.0.1",
 		[]corev1.ServicePort{{Name: "http", Port: 1234}},
@@ -830,9 +944,16 @@ func TestRevisionDeleted(t *testing.T) {
 	ei := fakeendpointsinformer.Get(ctx)
 	ep := ep(testRevision, 1234, "http", "128.0.0.1")
 	fakekubeclient.Get(ctx).CoreV1().Endpoints(testNamespace).Create(ep)
-	controller.StartInformers(ctx.Done(), ei.Informer())
+	waitInformers, err := controller.RunInformers(ctx.Done(), ei.Informer())
+	if err != nil {
+		t.Fatalf("Failed to start informers: %v", err)
+	}
+	defer func() {
+		cancel()
+		waitInformers()
+	}()
 
-	rev := revision(types.NamespacedName{testNamespace, testRevision}, networking.ProtocolHTTP1)
+	rev := revisionCC1(types.NamespacedName{testNamespace, testRevision}, networking.ProtocolHTTP1)
 	fakeservingclient.Get(ctx).ServingV1alpha1().Revisions(testNamespace).Create(rev)
 	ri := fakerevisioninformer.Get(ctx)
 	ri.Informer().GetIndexer().Add(rev)
@@ -840,28 +961,144 @@ func TestRevisionDeleted(t *testing.T) {
 	fakeRT := activatortest.FakeRoundTripper{}
 	rt := network.RoundTripperFunc(fakeRT.RT)
 
-	rbm := NewRevisionBackendsManager(
-		ctx,
-		rt,
-		TestLogger(t))
+	rbm := newRevisionBackendsManager(ctx, rt)
 	// Make some movements.
 	ei.Informer().GetIndexer().Add(ep)
 	select {
-	case <-rbm.UpdateCh():
+	case <-rbm.updates():
 	case <-time.After(time.Second * 2):
 		t.Errorf("Timedout waiting for initial response")
 	}
 	// Now delete the endpoints.
 	fakekubeclient.Get(ctx).CoreV1().Endpoints(testNamespace).Delete(ep.Name, &metav1.DeleteOptions{})
 	select {
-	case r := <-rbm.UpdateCh():
-		if got, want := r.ClusterIPDest, ""; got != want {
-			t.Errorf(`ClusterIP = %s, want ""`, got)
-		}
-		if got, want := len(r.Dests), 0; got != want {
-			t.Errorf("DestsLen = %d, want: 0", got)
-		}
-	case <-time.After(time.Second * 2):
-		t.Errorf("Timedout waiting for initial response")
+	case r := <-rbm.updates():
+		t.Errorf("Unexpected update: %#v", r)
+	case <-time.After(time.Millisecond * 200):
+		// Wait to make sure the callbacks are executed.
 	}
+
+	cancel()
+	waitForRevisionBackedMananger(t, rbm)
+}
+
+func TestServiceDoesNotExist(t *testing.T) {
+	// Tests when the service is not available.
+	ctx, cancel, _ := rtesting.SetupFakeContextWithCancel(t)
+
+	ei := fakeendpointsinformer.Get(ctx)
+	eps := ep(testRevision, 1234, "http", "128.0.0.1")
+	fakekubeclient.Get(ctx).CoreV1().Endpoints(testNamespace).Create(eps)
+	waitInformers, err := controller.RunInformers(ctx.Done(), ei.Informer())
+	if err != nil {
+		t.Fatalf("Failed to start informers: %v", err)
+	}
+	defer func() {
+		cancel()
+		waitInformers()
+	}()
+
+	rev := revisionCC1(types.NamespacedName{testNamespace, testRevision}, networking.ProtocolHTTP1)
+	fakeservingclient.Get(ctx).ServingV1alpha1().Revisions(testNamespace).Create(rev)
+	ri := fakerevisioninformer.Get(ctx)
+	ri.Informer().GetIndexer().Add(rev)
+
+	// This will make sure we go to the cluster IP probing.
+	fakeRT := activatortest.FakeRoundTripper{
+		ExpectHost: testRevision,
+		ProbeHostResponses: map[string][]activatortest.FakeResponse{
+			// To ensure that if we fail, when we get into the second iteration
+			// of probing if the test is not yet complete, we store 2 items here.
+			"128.0.0.1:1234": {{
+				Err: errors.New("clusterIP transport error"),
+			}, {
+				Err: errors.New("clusterIP transport error"),
+			}, {
+				Err: errors.New("clusterIP transport error"),
+			}},
+		},
+	}
+	rt := network.RoundTripperFunc(fakeRT.RT)
+
+	rbm := newRevisionBackendsManager(ctx, rt)
+	// Make some movements to generate a checkDests call.
+	ei.Informer().GetIndexer().Add(eps)
+	select {
+	case x := <-rbm.updates():
+		// We can't probe endpoints (see RT above) and we can't get to probe
+		// cluster IP. But if the service is accessible then we will and probing will
+		// succeed since RT has no rules for that.
+		t.Errorf("Unexpected update, should have had none: %v", x)
+	case <-time.After(200 * time.Millisecond):
+	}
+
+	cancel()
+	waitForRevisionBackedMananger(t, rbm)
+}
+
+func TestServiceMoreThanOne(t *testing.T) {
+	// Tests when the service is not available.
+	ctx, cancel, _ := rtesting.SetupFakeContextWithCancel(t)
+
+	ei := fakeendpointsinformer.Get(ctx)
+	eps := ep(testRevision, 1234, "http", "128.0.0.1")
+	fakekubeclient.Get(ctx).CoreV1().Endpoints(testNamespace).Create(eps)
+	waitInformers, err := controller.RunInformers(ctx.Done(), ei.Informer())
+	if err != nil {
+		t.Fatalf("Failed to start informers: %v", err)
+	}
+	defer func() {
+		cancel()
+		waitInformers()
+	}()
+
+	rev := revisionCC1(types.NamespacedName{testNamespace, testRevision}, networking.ProtocolHTTP1)
+	fakeservingclient.Get(ctx).ServingV1alpha1().Revisions(testNamespace).Create(rev)
+	ri := fakerevisioninformer.Get(ctx)
+	ri.Informer().GetIndexer().Add(rev)
+
+	// Now let's create two!
+	for _, num := range []string{"11", "12"} {
+		svc := privateSKSService(
+			types.NamespacedName{testNamespace, testRevision},
+			"129.0.0."+num,
+			[]corev1.ServicePort{{Name: "http", Port: 1234}},
+		)
+		// Modify the name so both can be created.
+		svc.Name = svc.Name + num
+		fakekubeclient.Get(ctx).CoreV1().Services(testNamespace).Create(svc)
+		si := fakeserviceinformer.Get(ctx)
+		si.Informer().GetIndexer().Add(svc)
+	}
+
+	// Make sure fake probe failures ensue.
+	fakeRT := activatortest.FakeRoundTripper{
+		ExpectHost: testRevision,
+		ProbeHostResponses: map[string][]activatortest.FakeResponse{
+			// To ensure that if we fail, when we get into the second iteration
+			// of probing if the test is not yet complete, we store 2 items here.
+			"128.0.0.1:1234": {{
+				Err: errors.New("clusterIP transport error"),
+			}, {
+				Err: errors.New("clusterIP transport error"),
+			}, {
+				Err: errors.New("clusterIP transport error"),
+			}},
+		},
+	}
+
+	rt := network.RoundTripperFunc(fakeRT.RT)
+	rbm := newRevisionBackendsManager(ctx, rt)
+	ei.Informer().GetIndexer().Add(eps)
+	select {
+	case x := <-rbm.updates():
+		// We can't probe endpoints (see RT above) and we can't get to probe
+		// cluster IP. But if the service is accessible then we will and probing will
+		// succeed since RT has no rules for that.
+		t.Errorf("Unexpected update, should have had none: %v", x)
+	case <-time.After(200 * time.Millisecond):
+	}
+
+	cancel()
+	waitForRevisionBackedMananger(t, rbm)
 }
