@@ -26,38 +26,39 @@ import (
 	"knative.dev/pkg/apis"
 	"knative.dev/pkg/ptr"
 	"knative.dev/serving/pkg/apis/serving"
+	v1 "knative.dev/serving/pkg/apis/serving/v1"
 	routeconfig "knative.dev/serving/pkg/reconciler/route/config"
 )
 
 func TestTrafficTargetValidation(t *testing.T) {
 	tests := []struct {
 		name string
-		tt   *TrafficTarget
+		tt   *v1.TrafficTarget
 		want *apis.FieldError
 		wc   func(context.Context) context.Context
 	}{{
 		name: "valid with revisionName",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			RevisionName: "bar",
-			Percent:      12,
+			Percent:      ptr.Int64(12),
 		},
 		wc:   apis.WithinSpec,
 		want: nil,
 	}, {
 		name: "valid with revisionName and name (spec)",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			Tag:          "foo",
 			RevisionName: "bar",
-			Percent:      12,
+			Percent:      ptr.Int64(12),
 		},
 		wc:   apis.WithinSpec,
 		want: nil,
 	}, {
 		name: "valid with revisionName and name (status)",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			Tag:          "foo",
 			RevisionName: "bar",
-			Percent:      12,
+			Percent:      ptr.Int64(12),
 			URL: &apis.URL{
 				Scheme: "http",
 				Host:   "foo.bar.com",
@@ -67,162 +68,176 @@ func TestTrafficTargetValidation(t *testing.T) {
 		want: nil,
 	}, {
 		name: "invalid with revisionName and name (status)",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			Tag:          "foo",
 			RevisionName: "bar",
-			Percent:      12,
+			Percent:      ptr.Int64(12),
 		},
 		wc:   apis.WithinStatus,
 		want: apis.ErrMissingField("url"),
 	}, {
 		name: "invalid with bad revisionName",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			RevisionName: "b ar",
-			Percent:      12,
+			Percent:      ptr.Int64(12),
 		},
 		wc: apis.WithinSpec,
 		want: apis.ErrInvalidKeyName(
 			"b ar", "revisionName", "name part must consist of alphanumeric characters, '-', '_' or '.', and must start and end with an alphanumeric character (e.g. 'MyName',  or 'my.name',  or '123-abc', regex used for validation is '([A-Za-z0-9][-A-Za-z0-9_.]*)?[A-Za-z0-9]')"),
 	}, {
 		name: "valid with revisionName and latestRevision",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			RevisionName:   "bar",
 			LatestRevision: ptr.Bool(false),
-			Percent:        12,
+			Percent:        ptr.Int64(12),
 		},
 		wc:   apis.WithinSpec,
 		want: nil,
 	}, {
 		name: "invalid with revisionName and latestRevision (spec)",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			RevisionName:   "bar",
 			LatestRevision: ptr.Bool(true),
-			Percent:        12,
+			Percent:        ptr.Int64(12),
 		},
 		wc:   apis.WithinSpec,
-		want: apis.ErrInvalidValue(true, "latestRevision"),
+		want: apis.ErrGeneric(`may not set revisionName "bar" when latestRevision is true`, "latestRevision"),
 	}, {
 		name: "valid with revisionName and latestRevision (status)",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			RevisionName:   "bar",
 			LatestRevision: ptr.Bool(true),
-			Percent:        12,
+			Percent:        ptr.Int64(12),
 		},
 		wc:   apis.WithinStatus,
 		want: nil,
 	}, {
 		name: "valid with configurationName",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			ConfigurationName: "bar",
-			Percent:           37,
+			Percent:           ptr.Int64(37),
 		},
 		wc:   apis.WithinSpec,
 		want: nil,
 	}, {
 		name: "valid with configurationName and name (spec)",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			Tag:               "foo",
 			ConfigurationName: "bar",
-			Percent:           37,
+			Percent:           ptr.Int64(37),
 		},
 		wc:   apis.WithinSpec,
 		want: nil,
 	}, {
 		name: "invalid with bad configurationName",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			ConfigurationName: "b ar",
-			Percent:           37,
+			Percent:           ptr.Int64(37),
 		},
 		wc: apis.WithinSpec,
 		want: apis.ErrInvalidKeyName(
 			"b ar", "configurationName", "name part must consist of alphanumeric characters, '-', '_' or '.', and must start and end with an alphanumeric character (e.g. 'MyName',  or 'my.name',  or '123-abc', regex used for validation is '([A-Za-z0-9][-A-Za-z0-9_.]*)?[A-Za-z0-9]')"),
 	}, {
 		name: "valid with configurationName and latestRevision",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			ConfigurationName: "blah",
 			LatestRevision:    ptr.Bool(true),
-			Percent:           37,
+			Percent:           ptr.Int64(37),
 		},
 		wc:   apis.WithinSpec,
 		want: nil,
 	}, {
 		name: "invalid with configurationName and latestRevision",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			ConfigurationName: "blah",
 			LatestRevision:    ptr.Bool(false),
-			Percent:           37,
+			Percent:           ptr.Int64(37),
 		},
 		wc:   apis.WithinSpec,
-		want: apis.ErrInvalidValue(false, "latestRevision"),
+		want: apis.ErrGeneric(`may not set revisionName "" when latestRevision is false`, "latestRevision"),
 	}, {
 		name: "invalid with configurationName and default configurationName",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			ConfigurationName: "blah",
-			Percent:           37,
+			Percent:           ptr.Int64(37),
 		},
-		wc:   WithDefaultConfigurationName,
+		wc:   v1.WithDefaultConfigurationName,
 		want: apis.ErrDisallowedFields("configurationName"),
 	}, {
 		name: "valid with only default configurationName",
-		tt: &TrafficTarget{
-			Percent: 37,
+		tt: &v1.TrafficTarget{
+			Percent: ptr.Int64(37),
 		},
 		wc: func(ctx context.Context) context.Context {
-			return WithDefaultConfigurationName(apis.WithinSpec(ctx))
+			return v1.WithDefaultConfigurationName(apis.WithinSpec(ctx))
 		},
 		want: nil,
 	}, {
 		name: "valid with default configurationName and latestRevision",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			LatestRevision: ptr.Bool(true),
-			Percent:        37,
+			Percent:        ptr.Int64(37),
 		},
 		wc: func(ctx context.Context) context.Context {
-			return WithDefaultConfigurationName(apis.WithinSpec(ctx))
+			return v1.WithDefaultConfigurationName(apis.WithinSpec(ctx))
 		},
 		want: nil,
 	}, {
 		name: "invalid with default configurationName and latestRevision",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			LatestRevision: ptr.Bool(false),
-			Percent:        37,
+			Percent:        ptr.Int64(37),
 		},
 		wc: func(ctx context.Context) context.Context {
-			return WithDefaultConfigurationName(apis.WithinSpec(ctx))
+			return v1.WithDefaultConfigurationName(apis.WithinSpec(ctx))
 		},
-		want: apis.ErrInvalidValue(false, "latestRevision"),
+		want: apis.ErrGeneric(`may not set revisionName "" when latestRevision is false`, "latestRevision"),
 	}, {
 		name: "invalid without revisionName in status",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			ConfigurationName: "blah",
-			Percent:           37,
+			Percent:           ptr.Int64(37),
 		},
 		wc:   apis.WithinStatus,
 		want: apis.ErrMissingField("revisionName"),
 	}, {
 		name: "valid with revisionName and default configurationName",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			RevisionName: "bar",
-			Percent:      12,
+			Percent:      ptr.Int64(12),
 		},
-		wc:   WithDefaultConfigurationName,
+		wc:   v1.WithDefaultConfigurationName,
 		want: nil,
 	}, {
 		name: "valid with no percent",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			ConfigurationName: "booga",
 		},
 		want: nil,
+	}, {
+		name: "valid with nil percent",
+		tt: &v1.TrafficTarget{
+			ConfigurationName: "booga",
+			Percent:           nil,
+		},
+		want: nil,
+	}, {
+		name: "valid with zero percent",
+		tt: &v1.TrafficTarget{
+			ConfigurationName: "booga",
+			Percent:           ptr.Int64(0),
+		},
+		want: nil,
 	}, {
 		name: "valid with no name",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			ConfigurationName: "booga",
-			Percent:           100,
+			Percent:           ptr.Int64(100),
 		},
 		want: nil,
 	}, {
 		name: "invalid with both",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			RevisionName:      "foo",
 			ConfigurationName: "bar",
 		},
@@ -232,8 +247,8 @@ func TestTrafficTargetValidation(t *testing.T) {
 		},
 	}, {
 		name: "invalid with neither",
-		tt: &TrafficTarget{
-			Percent: 100,
+		tt: &v1.TrafficTarget{
+			Percent: ptr.Int64(100),
 		},
 		want: &apis.FieldError{
 			Message: "expected exactly one, got neither",
@@ -241,23 +256,23 @@ func TestTrafficTargetValidation(t *testing.T) {
 		},
 	}, {
 		name: "invalid percent too low",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			RevisionName: "foo",
-			Percent:      -5,
+			Percent:      ptr.Int64(-5),
 		},
 		want: apis.ErrOutOfBoundsValue("-5", "0", "100", "percent"),
 	}, {
 		name: "invalid percent too high",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			RevisionName: "foo",
-			Percent:      101,
+			Percent:      ptr.Int64(101),
 		},
 		want: apis.ErrOutOfBoundsValue("101", "0", "100", "percent"),
 	}, {
 		name: "disallowed url set",
-		tt: &TrafficTarget{
+		tt: &v1.TrafficTarget{
 			ConfigurationName: "foo",
-			Percent:           100,
+			Percent:           ptr.Int64(100),
 			URL: &apis.URL{
 				Host: "should.not.be.set",
 			},
@@ -292,19 +307,19 @@ func TestRouteValidation(t *testing.T) {
 			ObjectMeta: metav1.ObjectMeta{
 				Name: "valid",
 			},
-			Spec: RouteSpec{
-				Traffic: []TrafficTarget{{
+			Spec: v1.RouteSpec{
+				Traffic: []v1.TrafficTarget{{
 					Tag:          "bar",
 					RevisionName: "foo",
-					Percent:      100,
+					Percent:      ptr.Int64(100),
 				}},
 			},
-			Status: RouteStatus{
-				RouteStatusFields: RouteStatusFields{
-					Traffic: []TrafficTarget{{
+			Status: v1.RouteStatus{
+				RouteStatusFields: v1.RouteStatusFields{
+					Traffic: []v1.TrafficTarget{{
 						Tag:          "bar",
 						RevisionName: "foo",
-						Percent:      100,
+						Percent:      ptr.Int64(100),
 						URL: &apis.URL{
 							Scheme: "http",
 							Host:   "bar.blah.com",
@@ -320,15 +335,15 @@ func TestRouteValidation(t *testing.T) {
 			ObjectMeta: metav1.ObjectMeta{
 				Name: "valid",
 			},
-			Spec: RouteSpec{
-				Traffic: []TrafficTarget{{
+			Spec: v1.RouteSpec{
+				Traffic: []v1.TrafficTarget{{
 					Tag:          "prod",
 					RevisionName: "foo",
-					Percent:      90,
+					Percent:      ptr.Int64(90),
 				}, {
 					Tag:               "experiment",
 					ConfigurationName: "bar",
-					Percent:           10,
+					Percent:           ptr.Int64(10),
 				}},
 			},
 		},
@@ -339,19 +354,19 @@ func TestRouteValidation(t *testing.T) {
 			ObjectMeta: metav1.ObjectMeta{
 				Name: "valid",
 			},
-			Spec: RouteSpec{
-				Traffic: []TrafficTarget{{
+			Spec: v1.RouteSpec{
+				Traffic: []v1.TrafficTarget{{
 					Tag:          "bar",
 					RevisionName: "foo",
-					Percent:      100,
+					Percent:      ptr.Int64(100),
 				}},
 			},
-			Status: RouteStatus{
-				RouteStatusFields: RouteStatusFields{
-					Traffic: []TrafficTarget{{
+			Status: v1.RouteStatus{
+				RouteStatusFields: v1.RouteStatusFields{
+					Traffic: []v1.TrafficTarget{{
 						Tag:          "bar",
 						RevisionName: "foo",
-						Percent:      100,
+						Percent:      ptr.Int64(100),
 					}},
 				},
 			},
@@ -368,10 +383,10 @@ func TestRouteValidation(t *testing.T) {
 			ObjectMeta: metav1.ObjectMeta{
 				Name: "valid",
 			},
-			Spec: RouteSpec{
-				Traffic: []TrafficTarget{{
+			Spec: v1.RouteSpec{
+				Traffic: []v1.TrafficTarget{{
 					Tag:     "foo",
-					Percent: 100,
+					Percent: ptr.Int64(100),
 				}},
 			},
 		},
@@ -388,15 +403,15 @@ func TestRouteValidation(t *testing.T) {
 			ObjectMeta: metav1.ObjectMeta{
 				Name: "valid",
 			},
-			Spec: RouteSpec{
-				Traffic: []TrafficTarget{{
+			Spec: v1.RouteSpec{
+				Traffic: []v1.TrafficTarget{{
 					Tag:          "foo",
 					RevisionName: "bar",
-					Percent:      50,
+					Percent:      ptr.Int64(50),
 				}, {
 					Tag:          "foo",
 					RevisionName: "bar",
-					Percent:      50,
+					Percent:      ptr.Int64(50),
 				}},
 			},
 		},
@@ -413,10 +428,10 @@ func TestRouteValidation(t *testing.T) {
 			ObjectMeta: metav1.ObjectMeta{
 				Name: "do.not.use.dots",
 			},
-			Spec: RouteSpec{
-				Traffic: []TrafficTarget{{
+			Spec: v1.RouteSpec{
+				Traffic: []v1.TrafficTarget{{
 					RevisionName: "foo",
-					Percent:      100,
+					Percent:      ptr.Int64(100),
 				}},
 			},
 		},
@@ -430,10 +445,10 @@ func TestRouteValidation(t *testing.T) {
 			ObjectMeta: metav1.ObjectMeta{
 				Name: "do.not.use.dots",
 			},
-			Spec: RouteSpec{
-				Traffic: []TrafficTarget{{
+			Spec: v1.RouteSpec{
+				Traffic: []v1.TrafficTarget{{
 					RevisionName: "foo",
-					Percent:      90,
+					Percent:      ptr.Int64(90),
 				}},
 			},
 		},
@@ -450,10 +465,10 @@ func TestRouteValidation(t *testing.T) {
 			ObjectMeta: metav1.ObjectMeta{
 				Name: strings.Repeat("a", 64),
 			},
-			Spec: RouteSpec{
-				Traffic: []TrafficTarget{{
+			Spec: v1.RouteSpec{
+				Traffic: []v1.TrafficTarget{{
 					RevisionName: "foo",
-					Percent:      100,
+					Percent:      ptr.Int64(100),
 				}},
 			},
 		},
@@ -474,12 +489,12 @@ func TestRouteValidation(t *testing.T) {
 	}
 }
 
-func TestRouteLabelAnnotationValidation(t *testing.T) {
-	validRouteSpec := RouteSpec{
-		Traffic: []TrafficTarget{{
+func TestRouteLabelValidation(t *testing.T) {
+	validRouteSpec := v1.RouteSpec{
+		Traffic: []v1.TrafficTarget{{
 			Tag:          "bar",
 			RevisionName: "foo",
-			Percent:      100,
+			Percent:      ptr.Int64(100),
 		}},
 	}
 	tests := []struct {
@@ -616,3 +631,184 @@ func TestRouteLabelAnnotationValidation(t *testing.T) {
 		})
 	}
 }
+
+func getRouteSpec(confName string) v1.RouteSpec {
+	return v1.RouteSpec{
+		Traffic: []v1.TrafficTarget{{
+			LatestRevision:    ptr.Bool(true),
+			Percent:           ptr.Int64(100),
+			ConfigurationName: confName,
+		}},
+	}
+}
+
+func TestRouteAnnotationUpdate(t *testing.T) {
+	const (
+		u1 = "oveja@knative.dev"
+		u2 = "cabra@knative.dev"
+		u3 = "vaca@knative.dev"
+	)
+	tests := []struct {
+		name string
+		prev *Route
+		this *Route
+		want *apis.FieldError
+	}{{
+		name: "update creator annotation",
+		this: &Route{
+			ObjectMeta: metav1.ObjectMeta{
+				Name: "valid",
+				Annotations: map[string]string{
+					serving.CreatorAnnotation: u2,
+					serving.UpdaterAnnotation: u1,
+				},
+			},
+			Spec: getRouteSpec("old"),
+		},
+		prev: &Route{
+			ObjectMeta: metav1.ObjectMeta{
+				Name: "valid",
+				Annotations: map[string]string{
+					serving.CreatorAnnotation: u1,
+					serving.UpdaterAnnotation: u1,
+				},
+			},
+			Spec: getRouteSpec("old"),
+		},
+		want: (&apis.FieldError{Message: "annotation value is immutable",
+			Paths: []string{serving.CreatorAnnotation}}).ViaField("metadata.annotations"),
+	}, {
+		name: "update creator annotation with spec changes",
+		this: &Route{
+			ObjectMeta: metav1.ObjectMeta{
+				Name: "valid",
+				Annotations: map[string]string{
+					serving.CreatorAnnotation: u2,
+					serving.UpdaterAnnotation: u1,
+				},
+			},
+			Spec: getRouteSpec("new"),
+		},
+		prev: &Route{
+			ObjectMeta: metav1.ObjectMeta{
+				Name: "valid",
+				Annotations: map[string]string{
+					serving.CreatorAnnotation: u1,
+					serving.UpdaterAnnotation: u1,
+				},
+			},
+			Spec: getRouteSpec("old"),
+		},
+		want: (&apis.FieldError{Message: "annotation value is immutable",
+			Paths: []string{serving.CreatorAnnotation}}).ViaField("metadata.annotations"),
+	}, {
+		name: "update lastModifier without spec changes",
+		this: &Route{
+			ObjectMeta: metav1.ObjectMeta{
+				Name: "valid",
+				Annotations: map[string]string{
+					serving.CreatorAnnotation: u1,
+					serving.UpdaterAnnotation: u2,
+				},
+			},
+			Spec: getRouteSpec("old"),
+		},
+		prev: &Route{
+			ObjectMeta: metav1.ObjectMeta{
+				Name: "valid",
+				Annotations: map[string]string{
+					serving.CreatorAnnotation: u1,
+					serving.UpdaterAnnotation: u1,
+				},
+			},
+			Spec: getRouteSpec("old"),
+		},
+		want: apis.ErrInvalidValue(u2, serving.UpdaterAnnotation).ViaField("metadata.annotations"),
+	}, {
+		name: "update lastModifier with spec changes",
+		this: &Route{
+			ObjectMeta: metav1.ObjectMeta{
+				Name: "valid",
+				Annotations: map[string]string{
+					serving.CreatorAnnotation: u1,
+					serving.UpdaterAnnotation: u3,
+				},
+			},
+			Spec: getRouteSpec("new"),
+		},
+		prev: &Route{
+			ObjectMeta: metav1.ObjectMeta{
+				Name: "valid",
+				Annotations: map[string]string{
+					serving.CreatorAnnotation: u1,
+					serving.UpdaterAnnotation: u1,
+				},
+			},
+			Spec: getRouteSpec("old"),
+		},
+		want: nil,
+	}, {
+		name: "no validation for lastModifier annotation even after update as route owned by service",
+		this: &Route{
+			ObjectMeta: metav1.ObjectMeta{
+				Name: "valid",
+				Annotations: map[string]string{
+					serving.CreatorAnnotation: u1,
+					serving.UpdaterAnnotation: u3,
+				},
+				OwnerReferences: []metav1.OwnerReference{{
+					APIVersion: "v1beta1",
+					Kind:       serving.GroupName,
+				}},
+			},
+			Spec: getRouteSpec("old"),
+		},
+		prev: &Route{
+			ObjectMeta: metav1.ObjectMeta{
+				Name: "valid",
+				Annotations: map[string]string{
+					serving.CreatorAnnotation: u1,
+					serving.UpdaterAnnotation: u1,
+				},
+			},
+			Spec: getRouteSpec("old"),
+		},
+		want: nil,
+	}, {
+		name: "no validation for creator annotation even after update as route owned by service",
+		this: &Route{
+			ObjectMeta: metav1.ObjectMeta{
+				Name: "valid",
+				Annotations: map[string]string{
+					serving.CreatorAnnotation: u3,
+					serving.UpdaterAnnotation: u1,
+				},
+				OwnerReferences: []metav1.OwnerReference{{
+					APIVersion: "v1beta1",
+					Kind:       serving.GroupName,
+				}},
+			},
+			Spec: getRouteSpec("old"),
+		},
+		prev: &Route{
+			ObjectMeta: metav1.ObjectMeta{
+				Name: "valid",
+				Annotations: map[string]string{
+					serving.CreatorAnnotation: u1,
+					serving.UpdaterAnnotation: u1,
+				},
+			},
+			Spec: getRouteSpec("old"),
+		},
+		want: nil,
+	}}
+	for _, test := range tests {
+		t.Run(test.name, func(t *testing.T) {
+			ctx := context.Background()
+			ctx = apis.WithinUpdate(ctx, test.prev)
+			if diff := cmp.Diff(test.want.Error(), test.this.Validate(ctx).Error()); diff != "" {
+				t.Errorf("Validate (-want, +got) = %v", diff)
+			}
+		})
+	}
+}
