@@ -27,25 +27,25 @@ import (
 	"time"
 
 	// These are the fake informers we want setup.
-	fakedynamicclient "github.com/knative/pkg/injection/clients/dynamicclient/fake"
-	fakeservingclient "github.com/knative/serving/pkg/client/injection/client/fake"
+	fakedynamicclient "knative.dev/pkg/injection/clients/dynamicclient/fake"
+	fakeservingclient "knative.dev/serving/pkg/client/injection/client/fake"
+	podscalable "knative.dev/serving/pkg/client/injection/ducks/autoscaling/v1alpha1/podscalable/fake"
+
+	"knative.dev/pkg/apis"
+	"knative.dev/pkg/apis/duck"
+	_ "knative.dev/pkg/system/testing"
+	"knative.dev/serving/pkg/activator"
+	"knative.dev/serving/pkg/apis/autoscaling"
+	pav1alpha1 "knative.dev/serving/pkg/apis/autoscaling/v1alpha1"
+	nv1a1 "knative.dev/serving/pkg/apis/networking/v1alpha1"
+	"knative.dev/serving/pkg/apis/serving"
+	"knative.dev/serving/pkg/apis/serving/v1alpha1"
+	clientset "knative.dev/serving/pkg/client/clientset/versioned"
+	"knative.dev/serving/pkg/network"
+	"knative.dev/serving/pkg/reconciler/autoscaling/config"
+	revisionresources "knative.dev/serving/pkg/reconciler/revision/resources"
+	"knative.dev/serving/pkg/reconciler/revision/resources/names"
 
-	"github.com/knative/pkg/apis"
-	"github.com/knative/pkg/apis/duck"
-	"github.com/knative/pkg/logging"
-	logtesting "github.com/knative/pkg/logging/testing"
-	_ "github.com/knative/pkg/system/testing"
-	"github.com/knative/serving/pkg/activator"
-	"github.com/knative/serving/pkg/apis/autoscaling"
-	pav1alpha1 "github.com/knative/serving/pkg/apis/autoscaling/v1alpha1"
-	"github.com/knative/serving/pkg/apis/serving"
-	"github.com/knative/serving/pkg/apis/serving/v1alpha1"
-	clientset "github.com/knative/serving/pkg/client/clientset/versioned"
-	"github.com/knative/serving/pkg/network"
-	"github.com/knative/serving/pkg/reconciler/autoscaling/config"
-	revisionresources "github.com/knative/serving/pkg/reconciler/revision/resources"
-	"github.com/knative/serving/pkg/reconciler/revision/resources/names"
-	presources "github.com/knative/serving/pkg/resources"
 	v1 "k8s.io/api/apps/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
@@ -55,17 +55,17 @@ import (
 	fakedynamic "k8s.io/client-go/dynamic/fake"
 	clientgotesting "k8s.io/client-go/testing"
 
-	. "github.com/knative/pkg/reconciler/testing"
-	. "github.com/knative/serving/pkg/testing"
+	. "knative.dev/pkg/reconciler/testing"
+	. "knative.dev/serving/pkg/testing"
 )
 
 const (
 	testNamespace = "test-namespace"
 	testRevision  = "test-revision"
+	key           = testNamespace + "/" + testRevision
 )
 
 func TestScaler(t *testing.T) {
-	defer logtesting.ClearAll()
 	tests := []struct {
 		label               string
 		startReplicas       int
@@ -74,7 +74,8 @@ func TestScaler(t *testing.T) {
 		maxScale            int32
 		wantReplicas        int32
 		wantScaling         bool
-		kpaMutation         func(*pav1alpha1.PodAutoscaler)
+		sks                 SKSOption
+		paMutation          func(*pav1alpha1.PodAutoscaler)
 		proberfunc          func(*pav1alpha1.PodAutoscaler, http.RoundTripper) (bool, error)
 		wantCBCount         int
 		wantAsyncProbeCount int
@@ -84,18 +85,40 @@ func TestScaler(t *testing.T) {
 		scaleTo:       0,
 		wantReplicas:  1,
 		wantScaling:   false,
-		kpaMutation: func(k *pav1alpha1.PodAutoscaler) {
-			kpaMarkActive(k, time.Now().Add(-stableWindow).Add(1*time.Second))
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			paMarkActive(k, time.Now().Add(-stableWindow).Add(1*time.Second))
 		},
 		wantCBCount: 1,
+	}, {
+		// Custom window will be shorter in the tests with custom PA window.
+		label:         "waits to scale to zero (just before idle period), custom PA window",
+		startReplicas: 1,
+		scaleTo:       0,
+		wantReplicas:  1,
+		wantScaling:   false,
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			WithWindowAnnotation(paStableWindow.String())(k)
+			paMarkActive(k, time.Now().Add(-paStableWindow).Add(1*time.Second))
+		},
+		wantCBCount: 1,
+	}, {
+		label:         "custom PA window, check for standard window, no probe",
+		startReplicas: 1,
+		scaleTo:       0,
+		wantReplicas:  0,
+		wantScaling:   false,
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			WithWindowAnnotation(paStableWindow.String())(k)
+			paMarkActive(k, time.Now().Add(-stableWindow))
+		},
 	}, {
 		label:         "scale to 1 waiting for idle expires",
 		startReplicas: 10,
 		scaleTo:       0,
 		wantReplicas:  1,
 		wantScaling:   true,
-		kpaMutation: func(k *pav1alpha1.PodAutoscaler) {
-			kpaMarkActive(k, time.Now().Add(-stableWindow).Add(1*time.Second))
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			paMarkActive(k, time.Now().Add(-stableWindow).Add(1*time.Second))
 		},
 		wantCBCount: 1,
 	}, {
@@ -104,8 +127,18 @@ func TestScaler(t *testing.T) {
 		scaleTo:       0,
 		wantReplicas:  0,
 		wantScaling:   false,
-		kpaMutation: func(k *pav1alpha1.PodAutoscaler) {
-			kpaMarkActive(k, time.Now().Add(-stableWindow))
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			paMarkActive(k, time.Now().Add(-stableWindow))
+		},
+	}, {
+		label:         "waits to scale to zero after idle period (custom PA window)",
+		startReplicas: 1,
+		scaleTo:       0,
+		wantReplicas:  0,
+		wantScaling:   false,
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			WithWindowAnnotation(paStableWindow.String())(k)
+			paMarkActive(k, time.Now().Add(-paStableWindow))
 		},
 	}, {
 		label:         "scale to zero after grace period",
@@ -113,8 +146,8 @@ func TestScaler(t *testing.T) {
 		scaleTo:       0,
 		wantReplicas:  0,
 		wantScaling:   true,
-		kpaMutation: func(k *pav1alpha1.PodAutoscaler) {
-			kpaMarkInactive(k, time.Now().Add(-gracePeriod))
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			paMarkInactive(k, time.Now().Add(-gracePeriod))
 		},
 	}, {
 		label:         "waits to scale to zero (just before grace period)",
@@ -122,18 +155,43 @@ func TestScaler(t *testing.T) {
 		scaleTo:       0,
 		wantReplicas:  0,
 		wantScaling:   false,
-		kpaMutation: func(k *pav1alpha1.PodAutoscaler) {
-			kpaMarkInactive(k, time.Now().Add(-gracePeriod).Add(1*time.Second))
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			paMarkInactive(k, time.Now().Add(-gracePeriod).Add(time.Second))
 		},
 		wantCBCount: 1,
+	}, {
+		label:         "waits to scale to zero (just before grace period, sks short)",
+		startReplicas: 1,
+		scaleTo:       0,
+		wantReplicas:  0,
+		wantScaling:   false,
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			paMarkInactive(k, time.Now().Add(-gracePeriod).Add(time.Second))
+		},
+		sks: func(s *nv1a1.ServerlessService) {
+			markSKSInProxyFor(s, gracePeriod-time.Second)
+		},
+		wantCBCount: 1,
+	}, {
+		label:         "waits to scale to zero (just before grace period, sks in proxy long)",
+		startReplicas: 1,
+		scaleTo:       0,
+		wantReplicas:  0,
+		wantScaling:   true,
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			paMarkInactive(k, time.Now().Add(-gracePeriod).Add(time.Second))
+		},
+		sks: func(s *nv1a1.ServerlessService) {
+			markSKSInProxyFor(s, gracePeriod)
+		},
 	}, {
 		label:         "scale to zero after grace period, but fail prober",
 		startReplicas: 1,
 		scaleTo:       0,
 		wantReplicas:  0,
 		wantScaling:   false,
-		kpaMutation: func(k *pav1alpha1.PodAutoscaler) {
-			kpaMarkInactive(k, time.Now().Add(-gracePeriod))
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			paMarkInactive(k, time.Now().Add(-gracePeriod))
 		},
 		proberfunc: func(*pav1alpha1.PodAutoscaler, http.RoundTripper) (bool, error) {
 			return false, errors.New("hell or high water")
@@ -145,19 +203,29 @@ func TestScaler(t *testing.T) {
 		scaleTo:       0,
 		wantReplicas:  0,
 		wantScaling:   false,
-		kpaMutation: func(k *pav1alpha1.PodAutoscaler) {
-			kpaMarkInactive(k, time.Now().Add(-gracePeriod))
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			paMarkInactive(k, time.Now().Add(-gracePeriod))
 		},
 		proberfunc:          func(*pav1alpha1.PodAutoscaler, http.RoundTripper) (bool, error) { return false, nil },
 		wantAsyncProbeCount: 1,
 	}, {
-		label:         "does not scale while activating",
+		label:         "waits to scale to zero while activating until after deadline exceeded",
 		startReplicas: 1,
 		scaleTo:       0,
 		wantReplicas:  -1,
 		wantScaling:   false,
-		kpaMutation: func(k *pav1alpha1.PodAutoscaler) {
-			k.Status.MarkActivating("", "")
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			paMarkActivating(k, time.Now().Add(-activationTimeout/2))
+		},
+		wantCBCount: 1,
+	}, {
+		label:         "scale to zero while activating after deadline exceeded",
+		startReplicas: 1,
+		scaleTo:       0,
+		wantReplicas:  0,
+		wantScaling:   true,
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			paMarkActivating(k, time.Now().Add(-(activationTimeout + time.Second)))
 		},
 	}, {
 		label:         "scale down to minScale before grace period",
@@ -166,8 +234,9 @@ func TestScaler(t *testing.T) {
 		minScale:      2,
 		wantReplicas:  2,
 		wantScaling:   true,
-		kpaMutation: func(k *pav1alpha1.PodAutoscaler) {
-			kpaMarkInactive(k, time.Now().Add(-gracePeriod+time.Second))
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			paMarkInactive(k, time.Now().Add(-gracePeriod+time.Second))
+			WithReachabilityReachable(k)
 		},
 	}, {
 		label:         "scale down to minScale after grace period",
@@ -176,8 +245,31 @@ func TestScaler(t *testing.T) {
 		minScale:      2,
 		wantReplicas:  2,
 		wantScaling:   true,
-		kpaMutation: func(k *pav1alpha1.PodAutoscaler) {
-			kpaMarkInactive(k, time.Now().Add(-gracePeriod))
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			paMarkInactive(k, time.Now().Add(-gracePeriod))
+			WithReachabilityReachable(k)
+		},
+	}, {
+		label:         "ignore minScale if unreachable",
+		startReplicas: 10,
+		scaleTo:       0,
+		minScale:      2,
+		wantReplicas:  0,
+		wantScaling:   true,
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			paMarkInactive(k, time.Now().Add(-gracePeriod))
+			WithReachabilityUnreachable(k) // not needed, here for clarity
+		},
+	}, {
+		label:         "observe minScale if reachability unknown",
+		startReplicas: 10,
+		scaleTo:       0,
+		minScale:      2,
+		wantReplicas:  2,
+		wantScaling:   true,
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			paMarkInactive(k, time.Now().Add(-gracePeriod))
+			WithReachabilityUnknown(k)
 		},
 	}, {
 		label:         "scales up",
@@ -198,8 +290,8 @@ func TestScaler(t *testing.T) {
 		scaleTo:       10,
 		wantReplicas:  10,
 		wantScaling:   true,
-		kpaMutation: func(k *pav1alpha1.PodAutoscaler) {
-			kpaMarkInactive(k, time.Now().Add(-gracePeriod/2))
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			paMarkInactive(k, time.Now().Add(-gracePeriod/2))
 		},
 	}, {
 		label:         "does not scale up from zero with no metrics",
@@ -207,6 +299,9 @@ func TestScaler(t *testing.T) {
 		scaleTo:       -1, // no metrics
 		wantReplicas:  -1,
 		wantScaling:   false,
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			paMarkInactive(k, time.Now())
+		},
 	}, {
 		label:         "scales up from zero to desired one",
 		startReplicas: 0,
@@ -225,6 +320,9 @@ func TestScaler(t *testing.T) {
 		scaleTo:       -1,
 		wantReplicas:  -1,
 		wantScaling:   false,
+		paMutation: func(k *pav1alpha1.PodAutoscaler) {
+			paMarkActive(k, time.Now())
+		},
 	}}
 
 	for _, test := range tests {
@@ -236,7 +334,7 @@ func TestScaler(t *testing.T) {
 			revision := newRevision(t, fakeservingclient.Get(ctx), test.minScale, test.maxScale)
 			deployment := newDeployment(t, dynamicClient, names.Deployment(revision), test.startReplicas)
 			cbCount := 0
-			revisionScaler := newScaler(ctx, presources.NewPodScalableInformerFactory(ctx), func(interface{}, time.Duration) {
+			revisionScaler := newScaler(ctx, podscalable.Get(ctx), func(interface{}, time.Duration) {
 				cbCount++
 			})
 			if test.proberfunc != nil {
@@ -261,12 +359,17 @@ func TestScaler(t *testing.T) {
 				})
 
 			pa := newKPA(t, fakeservingclient.Get(ctx), revision)
-			if test.kpaMutation != nil {
-				test.kpaMutation(pa)
+			if test.paMutation != nil {
+				test.paMutation(pa)
+			}
+
+			sks := sks("ns", "name")
+			if test.sks != nil {
+				test.sks(sks)
 			}
 
 			ctx = config.ToContext(ctx, defaultConfig())
-			desiredScale, err := revisionScaler.Scale(ctx, pa, test.scaleTo)
+			desiredScale, err := revisionScaler.Scale(ctx, pa, sks, test.scaleTo)
 			if err != nil {
 				t.Error("Scale got an unexpected error: ", err)
 			}
@@ -290,7 +393,6 @@ func TestScaler(t *testing.T) {
 }
 
 func TestDisableScaleToZero(t *testing.T) {
-	defer logtesting.ClearAll()
 	tests := []struct {
 		label         string
 		startReplicas int
@@ -343,15 +445,16 @@ func TestDisableScaleToZero(t *testing.T) {
 			deployment := newDeployment(t, dynamicClient, names.Deployment(revision), test.startReplicas)
 			revisionScaler := &scaler{
 				dynamicClient:     fakedynamicclient.Get(ctx),
-				logger:            logging.FromContext(ctx),
-				psInformerFactory: presources.NewPodScalableInformerFactory(ctx),
+				psInformerFactory: podscalable.Get(ctx),
 			}
 			pa := newKPA(t, fakeservingclient.Get(ctx), revision)
+			paMarkActive(pa, time.Now())
+			WithReachabilityReachable(pa)
 
 			conf := defaultConfig()
 			conf.Autoscaler.EnableScaleToZero = false
 			ctx = config.ToContext(ctx, conf)
-			desiredScale, err := revisionScaler.Scale(ctx, pa, test.scaleTo)
+			desiredScale, err := revisionScaler.Scale(ctx, pa, nil /*sks doesn't matter in this test*/, test.scaleTo)
 
 			if err != nil {
 				t.Error("Scale got an unexpected error: ", err)
@@ -370,7 +473,7 @@ func TestDisableScaleToZero(t *testing.T) {
 }
 
 func newKPA(t *testing.T, servingClient clientset.Interface, revision *v1alpha1.Revision) *pav1alpha1.PodAutoscaler {
-	pa := revisionresources.MakeKPA(revision)
+	pa := revisionresources.MakePA(revision)
 	pa.Status.InitializeConditions()
 	_, err := servingClient.AutoscalingV1alpha1().PodAutoscalers(testNamespace).Create(pa)
 	if err != nil {
@@ -393,9 +496,6 @@ func newRevision(t *testing.T, servingClient clientset.Interface, minScale, maxS
 			Name:        testRevision,
 			Annotations: annotations,
 		},
-		Spec: v1alpha1.RevisionSpec{
-			DeprecatedConcurrencyModel: "Multi",
-		},
 	}
 	rev, err := servingClient.ServingV1alpha1().Revisions(testNamespace).Create(rev)
 	if err != nil {
@@ -447,20 +547,27 @@ func newDeployment(t *testing.T, dynamicClient dynamic.Interface, name string, r
 	return deployment
 }
 
-func kpaMarkActive(pa *pav1alpha1.PodAutoscaler, ltt time.Time) {
+func paMarkActive(pa *pav1alpha1.PodAutoscaler, ltt time.Time) {
 	pa.Status.MarkActive()
 
 	// This works because the conditions are sorted alphabetically
 	pa.Status.Conditions[0].LastTransitionTime = apis.VolatileTime{Inner: metav1.NewTime(ltt)}
 }
 
-func kpaMarkInactive(pa *pav1alpha1.PodAutoscaler, ltt time.Time) {
+func paMarkInactive(pa *pav1alpha1.PodAutoscaler, ltt time.Time) {
 	pa.Status.MarkInactive("", "")
 
 	// This works because the conditions are sorted alphabetically
 	pa.Status.Conditions[0].LastTransitionTime = apis.VolatileTime{Inner: metav1.NewTime(ltt)}
 }
 
+func paMarkActivating(pa *pav1alpha1.PodAutoscaler, ltt time.Time) {
+	pa.Status.MarkActivating("", "")
+
+	// This works because the conditions are sorted alphabetically
+	pa.Status.Conditions[0].LastTransitionTime = apis.VolatileTime{Inner: metav1.NewTime(ltt)}
+}
+
 func checkReplicas(t *testing.T, dynamicClient *fakedynamic.FakeDynamicClient, deployment *v1.Deployment, expectedScale int32) {
 	t.Helper()
 
@@ -515,6 +622,7 @@ func TestActivatorProbe(t *testing.T) {
 			return rsp.Result(), nil
 		},
 		wantRes: false,
+		wantErr: true,
 	}, {
 		name: "wrong body",
 		rt: func(r *http.Request) (*http.Response, error) {
@@ -523,6 +631,7 @@ func TestActivatorProbe(t *testing.T) {
 			return rsp.Result(), nil
 		},
 		wantRes: false,
+		wantErr: true,
 	}, {
 		name: "all wrong",
 		rt: func(r *http.Request) (*http.Response, error) {
@@ -539,7 +648,7 @@ func TestActivatorProbe(t *testing.T) {
 				t.Errorf("Result = %v, want: %v", got, want)
 			}
 			if got, want := err != nil, test.wantErr; got != want {
-				t.Errorf("WantErr = %v, want: %v", got, want)
+				t.Errorf("WantErr = %v, want: %v: actual error is: %v", got, want, err)
 			}
 		})
 	}
@@ -549,7 +658,13 @@ type countingProber struct {
 	count int
 }
 
-func (c *countingProber) Offer(ctx context.Context, target, headerValue string, arg interface{}, period, timeout time.Duration) bool {
+func (c *countingProber) Offer(ctx context.Context, target string, arg interface{}, period, timeout time.Duration, ops ...interface{}) bool {
 	c.count++
 	return true
 }
+
+func markSKSInProxyFor(sks *nv1a1.ServerlessService, d time.Duration) {
+	sks.Status.MarkActivatorEndpointsPopulated()
+	// This works because the conditions are sorted alphabetically
+	sks.Status.Conditions[0].LastTransitionTime = apis.VolatileTime{Inner: metav1.NewTime(time.Now().Add(-d))}
+}
