@@ -23,11 +23,11 @@ import (
 	"strings"
 
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/apimachinery/pkg/util/intstr"
 	"k8s.io/apimachinery/pkg/util/sets"
 
-	istiov1alpha1 "knative.dev/pkg/apis/istio/common/v1alpha1"
-	"knative.dev/pkg/apis/istio/v1alpha3"
+	"github.com/gogo/protobuf/types"
+	istiov1alpha3 "istio.io/api/networking/v1alpha3"
+	"istio.io/client-go/pkg/apis/networking/v1alpha3"
 	"knative.dev/pkg/kmeta"
 	"knative.dev/pkg/network"
 	"knative.dev/pkg/system"
@@ -40,7 +40,7 @@ import (
 
 // VirtualServiceNamespace gives the namespace of the child
 // VirtualServices for a given Ingress.
-func VirtualServiceNamespace(ia v1alpha1.IngressAccessor) string {
+func VirtualServiceNamespace(ia *v1alpha1.Ingress) string {
 	if len(ia.GetNamespace()) == 0 {
 		return system.Namespace()
 	}
@@ -49,7 +49,7 @@ func VirtualServiceNamespace(ia v1alpha1.IngressAccessor) string {
 
 // MakeIngressVirtualService creates Istio VirtualService as network
 // programming for Istio Gateways other than 'mesh'.
-func MakeIngressVirtualService(ia v1alpha1.IngressAccessor, gateways map[v1alpha1.IngressVisibility]sets.String) *v1alpha3.VirtualService {
+func MakeIngressVirtualService(ia *v1alpha1.Ingress, gateways map[v1alpha1.IngressVisibility]sets.String) *v1alpha3.VirtualService {
 	vs := &v1alpha3.VirtualService{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:            names.IngressVirtualService(ia),
@@ -72,7 +72,7 @@ func MakeIngressVirtualService(ia v1alpha1.IngressAccessor, gateways map[v1alpha
 }
 
 // MakeMeshVirtualService creates a mesh Virtual Service
-func MakeMeshVirtualService(ia v1alpha1.IngressAccessor) *v1alpha3.VirtualService {
+func MakeMeshVirtualService(ia *v1alpha1.Ingress) *v1alpha3.VirtualService {
 	vs := &v1alpha3.VirtualService{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:            names.MeshVirtualService(ia),
@@ -83,7 +83,7 @@ func MakeMeshVirtualService(ia v1alpha1.IngressAccessor) *v1alpha3.VirtualServic
 		Spec: *makeVirtualServiceSpec(ia, map[v1alpha1.IngressVisibility]sets.String{
 			v1alpha1.IngressVisibilityExternalIP:   sets.NewString("mesh"),
 			v1alpha1.IngressVisibilityClusterLocal: sets.NewString("mesh"),
-		}, keepLocalHostnames(getHosts(ia))),
+		}, expandedHosts(keepLocalHostnames(getHosts(ia)))),
 	}
 	vs.Labels = resources.FilterMap(ia.GetLabels(), func(k string) bool {
 		return k != serving.RouteLabelKey && k != serving.RouteNamespaceLabelKey
@@ -93,11 +93,11 @@ func MakeMeshVirtualService(ia v1alpha1.IngressAccessor) *v1alpha3.VirtualServic
 }
 
 // MakeVirtualServices creates a mesh virtualservice and a virtual service for each gateway
-func MakeVirtualServices(ia v1alpha1.IngressAccessor, gateways map[v1alpha1.IngressVisibility]sets.String) ([]*v1alpha3.VirtualService, error) {
+func MakeVirtualServices(ia *v1alpha1.Ingress, gateways map[v1alpha1.IngressVisibility]sets.String) ([]*v1alpha3.VirtualService, error) {
 	// Insert probe header
-	ia = ia.DeepCopyObject().(v1alpha1.IngressAccessor)
+	ia = ia.DeepCopy()
 	if _, err := InsertProbe(ia); err != nil {
-		return nil, fmt.Errorf("failed to insert a probe into the IngressAccessor: %w", err)
+		return nil, fmt.Errorf("failed to insert a probe into the Ingress: %w", err)
 	}
 
 	vss := []*v1alpha3.VirtualService{MakeMeshVirtualService(ia)}
@@ -120,14 +120,14 @@ func MakeVirtualServices(ia v1alpha1.IngressAccessor, gateways map[v1alpha1.Ingr
 
 // InsertProbe adds a AppendHeader rule so that any request going through a Gateway is tagged with
 // the version of the Ingress currently deployed on the Gateway.
-func InsertProbe(ia v1alpha1.IngressAccessor) (string, error) {
+func InsertProbe(ia *v1alpha1.Ingress) (string, error) {
 	bytes, err := ComputeIngressHash(ia)
 	if err != nil {
-		return "", fmt.Errorf("failed to compute the hash of the IngressAccessor: %w", err)
+		return "", fmt.Errorf("failed to compute the hash of the Ingress: %w", err)
 	}
 	hash := fmt.Sprintf("%x", bytes)
 
-	for _, rule := range ia.GetSpec().Rules {
+	for _, rule := range ia.Spec.Rules {
 		for i := range rule.HTTP.Paths {
 			if rule.HTTP.Paths[i].AppendHeaders == nil {
 				rule.HTTP.Paths[i].AppendHeaders = make(map[string]string, 1)
@@ -140,10 +140,10 @@ func InsertProbe(ia v1alpha1.IngressAccessor) (string, error) {
 }
 
 // ComputeIngressHash computes a hash of the Ingress Spec, Namespace and Name
-func ComputeIngressHash(ia v1alpha1.IngressAccessor) ([16]byte, error) {
-	bytes, err := json.Marshal(ia.GetSpec())
+func ComputeIngressHash(ia *v1alpha1.Ingress) ([16]byte, error) {
+	bytes, err := json.Marshal(ia.Spec)
 	if err != nil {
-		return [16]byte{}, fmt.Errorf("failed to serialize IngressAccessor: %w", err)
+		return [16]byte{}, fmt.Errorf("failed to serialize Ingress: %w", err)
 	}
 	bytes = append(bytes, []byte(ia.GetNamespace())...)
 	bytes = append(bytes, []byte(ia.GetName())...)
@@ -151,9 +151,9 @@ func ComputeIngressHash(ia v1alpha1.IngressAccessor) ([16]byte, error) {
 }
 
 // HostsPerGateway returns the set of hosts that each Gateway handles
-func HostsPerGateway(ia v1alpha1.IngressAccessor, gateways map[v1alpha1.IngressVisibility]sets.String) map[string][]string {
+func HostsPerGateway(ia *v1alpha1.Ingress, gateways map[v1alpha1.IngressVisibility]sets.String) map[string][]string {
 	output := make(map[string][]string)
-	for _, rule := range ia.GetSpec().Rules {
+	for _, rule := range ia.Spec.Rules {
 		for host := range expandedHosts(sets.NewString(rule.Hosts...)) {
 			for gateway := range gateways[rule.Visibility] {
 				output[gateway] = append(output[gateway], host)
@@ -163,37 +163,26 @@ func HostsPerGateway(ia v1alpha1.IngressAccessor, gateways map[v1alpha1.IngressV
 	return output
 }
 
-func makeVirtualServiceSpec(ia v1alpha1.IngressAccessor, gateways map[v1alpha1.IngressVisibility]sets.String, hosts sets.String) *v1alpha3.VirtualServiceSpec {
+func makeVirtualServiceSpec(ia *v1alpha1.Ingress, gateways map[v1alpha1.IngressVisibility]sets.String, hosts sets.String) *istiov1alpha3.VirtualService {
 	gw := sets.String{}.Union(gateways[v1alpha1.IngressVisibilityClusterLocal]).Union(gateways[v1alpha1.IngressVisibilityExternalIP])
-	spec := v1alpha3.VirtualServiceSpec{
+	spec := istiov1alpha3.VirtualService{
 		Gateways: gw.List(),
 		Hosts:    hosts.List(),
 	}
 
-	for _, rule := range ia.GetSpec().Rules {
+	for _, rule := range ia.Spec.Rules {
 		for _, p := range rule.HTTP.Paths {
 			hosts := hosts.Intersection(sets.NewString(rule.Hosts...))
 			if hosts.Len() != 0 {
-				spec.HTTP = append(spec.HTTP, *makeVirtualServiceRoute(hosts, &p, gateways, rule.Visibility))
+				spec.Http = append(spec.Http, makeVirtualServiceRoute(hosts, &p, gateways, rule.Visibility))
 			}
 		}
 	}
 	return &spec
 }
 
-func makePortSelector(ios intstr.IntOrString) v1alpha3.PortSelector {
-	if ios.Type == intstr.Int {
-		return v1alpha3.PortSelector{
-			Number: uint32(ios.IntValue()),
-		}
-	}
-	return v1alpha3.PortSelector{
-		Name: ios.String(),
-	}
-}
-
-func makeVirtualServiceRoute(hosts sets.String, http *v1alpha1.HTTPIngressPath, gateways map[v1alpha1.IngressVisibility]sets.String, visibility v1alpha1.IngressVisibility) *v1alpha3.HTTPRoute {
-	matches := []v1alpha3.HTTPMatchRequest{}
+func makeVirtualServiceRoute(hosts sets.String, http *v1alpha1.HTTPIngressPath, gateways map[v1alpha1.IngressVisibility]sets.String, visibility v1alpha1.IngressVisibility) *istiov1alpha3.HTTPRoute {
+	matches := []*istiov1alpha3.HTTPMatchRequest{}
 	clusterDomainName := network.GetClusterDomainName()
 	for _, host := range hosts.List() {
 		g := gateways[visibility]
@@ -203,46 +192,48 @@ func makeVirtualServiceRoute(hosts sets.String, http *v1alpha1.HTTPIngressPath,
 		}
 		matches = append(matches, makeMatch(host, http.Path, g))
 	}
-	weights := []v1alpha3.HTTPRouteDestination{}
+	weights := []*istiov1alpha3.HTTPRouteDestination{}
 	for _, split := range http.Splits {
 
-		var h *v1alpha3.Headers
+		var h *istiov1alpha3.Headers
 
 		if len(split.AppendHeaders) > 0 {
-			h = &v1alpha3.Headers{
-				Request: &v1alpha3.HeaderOperations{
+			h = &istiov1alpha3.Headers{
+				Request: &istiov1alpha3.Headers_HeaderOperations{
 					Add: split.AppendHeaders,
 				},
 			}
 		}
 
-		weights = append(weights, v1alpha3.HTTPRouteDestination{
-			Destination: v1alpha3.Destination{
+		weights = append(weights, &istiov1alpha3.HTTPRouteDestination{
+			Destination: &istiov1alpha3.Destination{
 				Host: network.GetServiceHostname(
 					split.ServiceName, split.ServiceNamespace),
-				Port: makePortSelector(split.ServicePort),
+				Port: &istiov1alpha3.PortSelector{
+					Number: uint32(split.ServicePort.IntValue()),
+				},
 			},
-			Weight:  split.Percent,
+			Weight:  int32(split.Percent),
 			Headers: h,
 		})
 	}
 
-	var h *v1alpha3.Headers
+	var h *istiov1alpha3.Headers
 	if len(http.AppendHeaders) > 0 {
-		h = &v1alpha3.Headers{
-			Request: &v1alpha3.HeaderOperations{
+		h = &istiov1alpha3.Headers{
+			Request: &istiov1alpha3.Headers_HeaderOperations{
 				Add: http.AppendHeaders,
 			},
 		}
 	}
 
-	return &v1alpha3.HTTPRoute{
+	return &istiov1alpha3.HTTPRoute{
 		Match:   matches,
 		Route:   weights,
-		Timeout: http.Timeout.Duration.String(),
-		Retries: &v1alpha3.HTTPRetry{
-			Attempts:      http.Retries.Attempts,
-			PerTryTimeout: http.Retries.PerTryTimeout.Duration.String(),
+		Timeout: types.DurationProto(http.Timeout.Duration),
+		Retries: &istiov1alpha3.HTTPRetry{
+			Attempts:      int32(http.Retries.Attempts),
+			PerTryTimeout: types.DurationProto(http.Retries.PerTryTimeout.Duration),
 		},
 		Headers:          h,
 		WebsocketUpgrade: true,
@@ -277,19 +268,19 @@ func expandedHosts(hosts sets.String) sets.String {
 	return expanded
 }
 
-func makeMatch(host string, pathRegExp string, gateways sets.String) v1alpha3.HTTPMatchRequest {
-	match := v1alpha3.HTTPMatchRequest{
+func makeMatch(host string, pathRegExp string, gateways sets.String) *istiov1alpha3.HTTPMatchRequest {
+	match := &istiov1alpha3.HTTPMatchRequest{
 		Gateways: gateways.List(),
-		Authority: &istiov1alpha1.StringMatch{
+		Authority: &istiov1alpha3.StringMatch{
 			// Do not use Regex as Istio 1.4 or later has 100 bytes limitation.
-			Prefix: hostPrefix(host),
+			MatchType: &istiov1alpha3.StringMatch_Prefix{Prefix: hostPrefix(host)},
 		},
 	}
 	// Empty pathRegExp is considered match all path. We only need to
 	// consider pathRegExp when it's non-empty.
 	if pathRegExp != "" {
-		match.URI = &istiov1alpha1.StringMatch{
-			Regex: pathRegExp,
+		match.Uri = &istiov1alpha3.StringMatch{
+			MatchType: &istiov1alpha3.StringMatch_Regex{Regex: pathRegExp},
 		}
 	}
 	return match
@@ -305,21 +296,17 @@ func hostPrefix(host string) string {
 	return strings.TrimSuffix(host, localDomainSuffix)
 }
 
-func optional(regexp string) string {
-	return "(" + regexp + ")?"
-}
-
-func getHosts(ia v1alpha1.IngressAccessor) sets.String {
+func getHosts(ia *v1alpha1.Ingress) sets.String {
 	hosts := sets.NewString()
-	for _, rule := range ia.GetSpec().Rules {
+	for _, rule := range ia.Spec.Rules {
 		hosts.Insert(rule.Hosts...)
 	}
 	return hosts
 }
 
-func getClusterLocalIngressRules(ci v1alpha1.IngressAccessor) []v1alpha1.IngressRule {
+func getClusterLocalIngressRules(i *v1alpha1.Ingress) []v1alpha1.IngressRule {
 	var result []v1alpha1.IngressRule
-	for _, rule := range ci.GetSpec().Rules {
+	for _, rule := range i.Spec.Rules {
 		if rule.Visibility == v1alpha1.IngressVisibilityClusterLocal {
 			result = append(result, rule)
 		}
@@ -328,9 +315,9 @@ func getClusterLocalIngressRules(ci v1alpha1.IngressAccessor) []v1alpha1.Ingress
 	return result
 }
 
-func getPublicIngressRules(ci v1alpha1.IngressAccessor) []v1alpha1.IngressRule {
+func getPublicIngressRules(i *v1alpha1.Ingress) []v1alpha1.IngressRule {
 	var result []v1alpha1.IngressRule
-	for _, rule := range ci.GetSpec().Rules {
+	for _, rule := range i.Spec.Rules {
 		if rule.Visibility == v1alpha1.IngressVisibilityExternalIP || rule.Visibility == "" {
 			result = append(result, rule)
 		}
