@@ -17,29 +17,30 @@ limitations under the License.
 package resources
 
 import (
+	"fmt"
 	"testing"
 
 	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-cmp/cmp/cmpopts"
-	"github.com/knative/pkg/logging"
-	pkgmetrics "github.com/knative/pkg/metrics"
-	_ "github.com/knative/pkg/metrics/testing"
-	"github.com/knative/pkg/ptr"
-	"github.com/knative/pkg/system"
-	_ "github.com/knative/pkg/system/testing"
-	"github.com/knative/serving/pkg/apis/networking"
-	"github.com/knative/serving/pkg/apis/serving"
-	"github.com/knative/serving/pkg/apis/serving/v1alpha1"
-	"github.com/knative/serving/pkg/apis/serving/v1beta1"
-	"github.com/knative/serving/pkg/autoscaler"
-	"github.com/knative/serving/pkg/deployment"
-	"github.com/knative/serving/pkg/metrics"
-	"github.com/knative/serving/pkg/network"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/intstr"
+	"knative.dev/pkg/logging"
+	pkgmetrics "knative.dev/pkg/metrics"
+	_ "knative.dev/pkg/metrics/testing"
+	"knative.dev/pkg/ptr"
+	"knative.dev/pkg/system"
+	_ "knative.dev/pkg/system/testing"
+	tracingconfig "knative.dev/pkg/tracing/config"
+	"knative.dev/serving/pkg/apis/networking"
+	"knative.dev/serving/pkg/apis/serving"
+	v1 "knative.dev/serving/pkg/apis/serving/v1"
+	"knative.dev/serving/pkg/apis/serving/v1alpha1"
+	"knative.dev/serving/pkg/deployment"
+	"knative.dev/serving/pkg/metrics"
+	"knative.dev/serving/pkg/network"
 )
 
 var (
@@ -69,10 +70,18 @@ var (
 	}
 
 	defaultQueueContainer = &corev1.Container{
-		Name:            QueueContainerName,
-		Resources:       createQueueResources(make(map[string]string), &corev1.Container{}),
-		Ports:           append(queueNonServingPorts, queueHTTPPort),
-		ReadinessProbe:  queueReadinessProbe,
+		Name:      QueueContainerName,
+		Resources: createQueueResources(make(map[string]string), &corev1.Container{}),
+		Ports:     append(queueNonServingPorts, queueHTTPPort),
+		ReadinessProbe: &corev1.Probe{
+			Handler: corev1.Handler{
+				Exec: &corev1.ExecAction{
+					Command: []string{"/ko-app/queue", "-probe-period", "0"},
+				},
+			},
+			PeriodSeconds:  1,
+			TimeoutSeconds: 10,
+		},
 		SecurityContext: queueSecurityContext,
 		Env: []corev1.EnvVar{{
 			Name:  "SERVING_NAMESPACE",
@@ -117,6 +126,21 @@ var (
 		}, {
 			Name:  "SERVING_REQUEST_METRICS_BACKEND",
 			Value: "",
+		}, {
+			Name:  "TRACING_CONFIG_BACKEND",
+			Value: "",
+		}, {
+			Name:  "TRACING_CONFIG_ZIPKIN_ENDPOINT",
+			Value: "",
+		}, {
+			Name:  "TRACING_CONFIG_STACKDRIVER_PROJECT_ID",
+			Value: "",
+		}, {
+			Name:  "TRACING_CONFIG_DEBUG",
+			Value: "false",
+		}, {
+			Name:  "TRACING_CONFIG_SAMPLE_RATE",
+			Value: "0.000000",
 		}, {
 			Name:  "USER_PORT",
 			Value: "8080",
@@ -138,6 +162,15 @@ var (
 		}, {
 			Name:  "INTERNAL_VOLUME_PATH",
 			Value: internalVolumePath,
+		}, {
+			Name:  "SERVING_READINESS_PROBE",
+			Value: fmt.Sprintf(`{"tcpSocket":{"port":%d,"host":"127.0.0.1"}}`, v1alpha1.DefaultUserPort),
+		}, {
+			Name:  "ENABLE_PROFILING",
+			Value: "false",
+		}, {
+			Name:  "SERVING_ENABLE_PROBE_REQUEST_LOG",
+			Value: "false",
 		}},
 	}
 
@@ -172,7 +205,7 @@ var (
 					serving.RevisionUID: "1234",
 				},
 			},
-			ProgressDeadlineSeconds: &ProgressDeadlineSeconds,
+			ProgressDeadlineSeconds: ptr.Int32(ProgressDeadlineSeconds),
 			Template: corev1.PodTemplateSpec{
 				ObjectMeta: metav1.ObjectMeta{
 					Labels: map[string]string{
@@ -180,9 +213,7 @@ var (
 						serving.RevisionUID:      "1234",
 						AppLabelKey:              "bar",
 					},
-					Annotations: map[string]string{
-						sidecarIstioInjectAnnotation: "true",
-					},
+					Annotations: map[string]string{},
 				},
 				// Spec: filled in by makePodSpec
 			},
@@ -197,6 +228,7 @@ var (
 			Labels: map[string]string{
 				serving.ConfigurationLabelKey: "cfg",
 				serving.ServiceLabelKey:       "svc",
+				serving.RouteLabelKey:         "im-a-route",
 			},
 		},
 		Spec: v1alpha1.RevisionSpec{
@@ -204,7 +236,7 @@ var (
 				Name:  containerName,
 				Image: "busybox",
 			},
-			RevisionSpec: v1beta1.RevisionSpec{
+			RevisionSpec: v1.RevisionSpec{
 				TimeoutSeconds: ptr.Int64(45),
 			},
 		},
@@ -263,6 +295,15 @@ func withReadinessProbe(handler corev1.Handler) containerOption {
 	}
 }
 
+func withTCPReadinessProbe() containerOption {
+	return withReadinessProbe(corev1.Handler{
+		TCPSocket: &corev1.TCPSocketAction{
+			Host: "127.0.0.1",
+			Port: intstr.FromInt(v1alpha1.DefaultUserPort),
+		},
+	})
+}
+
 func withHTTPReadinessProbe(port int) containerOption {
 	return withReadinessProbe(corev1.Handler{
 		HTTPGet: &corev1.HTTPGetAction{
@@ -272,19 +313,6 @@ func withHTTPReadinessProbe(port int) containerOption {
 	})
 }
 
-func withHTTPQPReadinessProbe(c *corev1.Container) {
-	withReadinessProbe(corev1.Handler{
-		HTTPGet: &corev1.HTTPGetAction{
-			Port: intstr.FromInt(networking.BackendHTTPPort),
-			Path: "/",
-			HTTPHeaders: []corev1.HTTPHeader{{
-				Name:  network.KubeletProbeHeaderName,
-				Value: "queue",
-			}},
-		},
-	})(c)
-}
-
 func withExecReadinessProbe(command []string) containerOption {
 	return withReadinessProbe(corev1.Handler{
 		Exec: &corev1.ExecAction{
@@ -338,9 +366,9 @@ func revision(opts ...revisionOption) *v1alpha1.Revision {
 	return revision
 }
 
-func withContainerConcurrency(cc v1beta1.RevisionContainerConcurrencyType) revisionOption {
+func withContainerConcurrency(cc int64) revisionOption {
 	return func(revision *v1alpha1.Revision) {
-		revision.Spec.ContainerConcurrency = cc
+		revision.Spec.ContainerConcurrency = &cc
 	}
 }
 
@@ -365,8 +393,8 @@ func TestMakePodSpec(t *testing.T) {
 		name string
 		rev  *v1alpha1.Revision
 		lc   *logging.Config
+		tc   *tracingconfig.Config
 		oc   *metrics.ObservabilityConfig
-		ac   *autoscaler.Config
 		cc   *deployment.Config
 		want *corev1.PodSpec
 	}{{
@@ -377,11 +405,14 @@ func TestMakePodSpec(t *testing.T) {
 				revision.Spec.GetContainer().Ports = []corev1.ContainerPort{{
 					ContainerPort: 8888,
 				}}
+				container(revision.Spec.GetContainer(),
+					withTCPReadinessProbe(),
+				)
 			},
 		),
 		lc: &logging.Config{},
+		tc: &tracingconfig.Config{},
 		oc: &metrics.ObservabilityConfig{},
-		ac: &autoscaler.Config{},
 		cc: &deployment.Config{},
 		want: podSpec(
 			[]corev1.Container{
@@ -394,6 +425,7 @@ func TestMakePodSpec(t *testing.T) {
 				queueContainer(
 					withEnvVar("CONTAINER_CONCURRENCY", "1"),
 					withEnvVar("USER_PORT", "8888"),
+					withEnvVar("SERVING_READINESS_PROBE", `{"tcpSocket":{"port":8888,"host":"127.0.0.1"}}`),
 				),
 			}),
 	}, {
@@ -408,6 +440,9 @@ func TestMakePodSpec(t *testing.T) {
 					Name:      "asdf",
 					MountPath: "/asdf",
 				}}
+				container(revision.Spec.GetContainer(),
+					withTCPReadinessProbe(),
+				)
 				revision.Spec.Volumes = []corev1.Volume{{
 					Name: "asdf",
 					VolumeSource: corev1.VolumeSource{
@@ -419,8 +454,8 @@ func TestMakePodSpec(t *testing.T) {
 			},
 		),
 		lc: &logging.Config{},
+		tc: &tracingconfig.Config{},
 		oc: &metrics.ObservabilityConfig{},
-		ac: &autoscaler.Config{},
 		cc: &deployment.Config{},
 		want: podSpec(
 			[]corev1.Container{
@@ -437,6 +472,7 @@ func TestMakePodSpec(t *testing.T) {
 				queueContainer(
 					withEnvVar("CONTAINER_CONCURRENCY", "1"),
 					withEnvVar("USER_PORT", "8888"),
+					withEnvVar("SERVING_READINESS_PROBE", `{"tcpSocket":{"port":8888,"host":"127.0.0.1"}}`),
 				),
 			}, withAppendedVolumes(corev1.Volume{
 				Name: "asdf",
@@ -448,11 +484,18 @@ func TestMakePodSpec(t *testing.T) {
 			})),
 	}, {
 		name: "concurrency=1 no owner",
-		rev:  revision(withContainerConcurrency(1)),
-		lc:   &logging.Config{},
-		oc:   &metrics.ObservabilityConfig{},
-		ac:   &autoscaler.Config{},
-		cc:   &deployment.Config{},
+		rev: revision(
+			withContainerConcurrency(1),
+			func(revision *v1alpha1.Revision) {
+				container(revision.Spec.GetContainer(),
+					withTCPReadinessProbe(),
+				)
+			},
+		),
+		lc: &logging.Config{},
+		tc: &tracingconfig.Config{},
+		oc: &metrics.ObservabilityConfig{},
+		cc: &deployment.Config{},
 		want: podSpec(
 			[]corev1.Container{
 				userContainer(),
@@ -468,11 +511,14 @@ func TestMakePodSpec(t *testing.T) {
 				revision.Status = v1alpha1.RevisionStatus{
 					ImageDigest: "busybox@sha256:deadbeef",
 				}
+				container(revision.Spec.GetContainer(),
+					withTCPReadinessProbe(),
+				)
 			},
 		),
 		lc: &logging.Config{},
+		tc: &tracingconfig.Config{},
 		oc: &metrics.ObservabilityConfig{},
-		ac: &autoscaler.Config{},
 		cc: &deployment.Config{},
 		want: podSpec(
 			[]corev1.Container{
@@ -488,10 +534,15 @@ func TestMakePodSpec(t *testing.T) {
 		rev: revision(
 			withContainerConcurrency(1),
 			withOwnerReference("parent-config"),
+			func(revision *v1alpha1.Revision) {
+				container(revision.Spec.GetContainer(),
+					withTCPReadinessProbe(),
+				)
+			},
 		),
 		lc: &logging.Config{},
+		tc: &tracingconfig.Config{},
 		oc: &metrics.ObservabilityConfig{},
-		ac: &autoscaler.Config{},
 		cc: &deployment.Config{},
 		want: podSpec(
 			[]corev1.Container{
@@ -509,46 +560,66 @@ func TestMakePodSpec(t *testing.T) {
 			)
 		}),
 		lc: &logging.Config{},
+		tc: &tracingconfig.Config{},
 		oc: &metrics.ObservabilityConfig{},
-		ac: &autoscaler.Config{},
 		cc: &deployment.Config{},
 		want: podSpec(
 			[]corev1.Container{
-				userContainer(
-					withHTTPQPReadinessProbe,
-				),
+				userContainer(),
 				queueContainer(
 					withEnvVar("CONTAINER_CONCURRENCY", "0"),
+					withEnvVar("SERVING_READINESS_PROBE", `{"httpGet":{"path":"/","port":8080,"host":"127.0.0.1","scheme":"HTTP","httpHeaders":[{"name":"K-Kubelet-Probe","value":"queue"}]}}`),
+				),
+			}),
+	}, {
+		name: "with tcp readiness probe",
+		rev: revision(func(revision *v1alpha1.Revision) {
+			container(revision.Spec.GetContainer(),
+				withReadinessProbe(corev1.Handler{
+					TCPSocket: &corev1.TCPSocketAction{
+						Host: "127.0.0.1",
+						Port: intstr.FromInt(12345),
+					},
+				}),
+			)
+		}),
+		lc: &logging.Config{},
+		tc: &tracingconfig.Config{},
+		oc: &metrics.ObservabilityConfig{},
+		cc: &deployment.Config{},
+		want: podSpec(
+			[]corev1.Container{
+				userContainer(),
+				queueContainer(
+					withEnvVar("CONTAINER_CONCURRENCY", "0"),
+					withEnvVar("SERVING_READINESS_PROBE", `{"tcpSocket":{"port":8080,"host":"127.0.0.1"}}`),
 				),
 			}),
 	}, {
 		name: "with shell readiness probe",
 		rev: revision(func(revision *v1alpha1.Revision) {
 			container(revision.Spec.GetContainer(),
-				withExecReadinessProbe(
-					[]string{"echo", "hello"},
-				),
+				withExecReadinessProbe([]string{"echo", "hello"}),
 			)
 		}),
 		lc: &logging.Config{},
+		tc: &tracingconfig.Config{},
 		oc: &metrics.ObservabilityConfig{},
-		ac: &autoscaler.Config{},
 		cc: &deployment.Config{},
 		want: podSpec(
 			[]corev1.Container{
 				userContainer(
-					withExecReadinessProbe(
-						[]string{"echo", "hello"},
-					),
-				),
+					withExecReadinessProbe([]string{"echo", "hello"})),
 				queueContainer(
 					withEnvVar("CONTAINER_CONCURRENCY", "0"),
+					withEnvVar("SERVING_READINESS_PROBE", "{}"),
 				),
 			}),
 	}, {
 		name: "with http liveness probe",
 		rev: revision(func(revision *v1alpha1.Revision) {
 			container(revision.Spec.GetContainer(),
+				withTCPReadinessProbe(),
 				withLivenessProbe(corev1.Handler{
 					HTTPGet: &corev1.HTTPGetAction{
 						Path: "/",
@@ -557,8 +628,8 @@ func TestMakePodSpec(t *testing.T) {
 			)
 		}),
 		lc: &logging.Config{},
+		tc: &tracingconfig.Config{},
 		oc: &metrics.ObservabilityConfig{},
-		ac: &autoscaler.Config{},
 		cc: &deployment.Config{},
 		want: podSpec(
 			[]corev1.Container{
@@ -582,14 +653,15 @@ func TestMakePodSpec(t *testing.T) {
 		name: "with tcp liveness probe",
 		rev: revision(func(revision *v1alpha1.Revision) {
 			container(revision.Spec.GetContainer(),
+				withTCPReadinessProbe(),
 				withLivenessProbe(corev1.Handler{
 					TCPSocket: &corev1.TCPSocketAction{},
 				}),
 			)
 		}),
 		lc: &logging.Config{},
+		tc: &tracingconfig.Config{},
 		oc: &metrics.ObservabilityConfig{},
-		ac: &autoscaler.Config{},
 		cc: &deployment.Config{},
 		want: podSpec(
 			[]corev1.Container{
@@ -606,12 +678,17 @@ func TestMakePodSpec(t *testing.T) {
 			}),
 	}, {
 		name: "with /var/log collection",
-		rev:  revision(withContainerConcurrency(1)),
-		lc:   &logging.Config{},
+		rev: revision(withContainerConcurrency(1),
+			func(revision *v1alpha1.Revision) {
+				container(revision.Spec.GetContainer(),
+					withTCPReadinessProbe(),
+				)
+			}),
+		lc: &logging.Config{},
+		tc: &tracingconfig.Config{},
 		oc: &metrics.ObservabilityConfig{
 			EnableVarLogCollection: true,
 		},
-		ac: &autoscaler.Config{},
 		cc: &deployment.Config{},
 		want: podSpec(
 			[]corev1.Container{
@@ -635,6 +712,7 @@ func TestMakePodSpec(t *testing.T) {
 				revision.Spec.GetContainer().Command = []string{"/bin/bash"}
 				revision.Spec.GetContainer().Args = []string{"-c", "echo Hello world"}
 				container(revision.Spec.GetContainer(),
+					withTCPReadinessProbe(),
 					withEnvVar("FOO", "bar"),
 					withEnvVar("BAZ", "blah"),
 				)
@@ -652,8 +730,8 @@ func TestMakePodSpec(t *testing.T) {
 			},
 		),
 		lc: &logging.Config{},
+		tc: &tracingconfig.Config{},
 		oc: &metrics.ObservabilityConfig{},
-		ac: &autoscaler.Config{},
 		cc: &deployment.Config{},
 		want: podSpec(
 			[]corev1.Container{
@@ -695,8 +773,10 @@ func TestMakePodSpec(t *testing.T) {
 			quantityComparer := cmp.Comparer(func(x, y resource.Quantity) bool {
 				return x.Cmp(y) == 0
 			})
-
-			got := makePodSpec(test.rev, test.lc, test.oc, test.ac, test.cc)
+			got, err := makePodSpec(test.rev, test.lc, test.tc, test.oc, test.cc)
+			if err != nil {
+				t.Fatal("makePodSpec returned errror")
+			}
 			if diff := cmp.Diff(test.want, got, quantityComparer); diff != "" {
 				t.Errorf("makePodSpec (-want, +got) = %v", diff)
 			}
@@ -713,7 +793,10 @@ func TestMakePodSpec(t *testing.T) {
 			}
 			test.rev.Spec.DeprecatedContainer = nil
 
-			got := makePodSpec(test.rev, test.lc, test.oc, test.ac, test.cc)
+			got, err := makePodSpec(test.rev, test.lc, test.tc, test.oc, test.cc)
+			if err != nil {
+				t.Fatal("makePodSpec returned errror")
+			}
 			if diff := cmp.Diff(test.want, got, quantityComparer); diff != "" {
 				t.Errorf("makePodSpec (-want, +got) = %v", diff)
 			}
@@ -721,14 +804,28 @@ func TestMakePodSpec(t *testing.T) {
 	}
 }
 
+func TestMissingProbeError(t *testing.T) {
+	_, err := MakeDeployment(defaultRevision,
+		&logging.Config{},
+		&tracingconfig.Config{},
+		&network.Config{},
+		&metrics.ObservabilityConfig{},
+		&deployment.Config{},
+	)
+
+	if err == nil {
+		t.Error("expected error from MakeDeployment")
+	}
+}
+
 func TestMakeDeployment(t *testing.T) {
 	tests := []struct {
 		name string
 		rev  *v1alpha1.Revision
 		lc   *logging.Config
+		tc   *tracingconfig.Config
 		nc   *network.Config
 		oc   *metrics.ObservabilityConfig
-		ac   *autoscaler.Config
 		cc   *deployment.Config
 		want *appsv1.Deployment
 	}{{
@@ -736,11 +833,21 @@ func TestMakeDeployment(t *testing.T) {
 		rev: revision(
 			withoutLabels,
 			withContainerConcurrency(1),
+			func(revision *v1alpha1.Revision) {
+				container(revision.Spec.GetContainer(),
+					withReadinessProbe(corev1.Handler{
+						TCPSocket: &corev1.TCPSocketAction{
+							Host: "127.0.0.1",
+							Port: intstr.FromInt(12345),
+						},
+					}),
+				)
+			},
 		),
 		lc:   &logging.Config{},
+		tc:   &tracingconfig.Config{},
 		nc:   &network.Config{},
 		oc:   &metrics.ObservabilityConfig{},
-		ac:   &autoscaler.Config{},
 		cc:   &deployment.Config{},
 		want: makeDeployment(),
 	}, {
@@ -748,22 +855,44 @@ func TestMakeDeployment(t *testing.T) {
 		rev: revision(
 			withoutLabels,
 			withOwnerReference("parent-config"),
+			func(revision *v1alpha1.Revision) {
+				container(revision.Spec.GetContainer(),
+					withReadinessProbe(corev1.Handler{
+						TCPSocket: &corev1.TCPSocketAction{
+							Host: "127.0.0.1",
+							Port: intstr.FromInt(12345),
+						},
+					}),
+				)
+			},
 		),
 		lc:   &logging.Config{},
+		tc:   &tracingconfig.Config{},
 		nc:   &network.Config{},
 		oc:   &metrics.ObservabilityConfig{},
-		ac:   &autoscaler.Config{},
 		cc:   &deployment.Config{},
 		want: makeDeployment(),
 	}, {
 		name: "with outbound IP range configured",
-		rev:  revision(withoutLabels),
-		lc:   &logging.Config{},
+		rev: revision(
+			withoutLabels,
+			func(revision *v1alpha1.Revision) {
+				container(revision.Spec.GetContainer(),
+					withReadinessProbe(corev1.Handler{
+						TCPSocket: &corev1.TCPSocketAction{
+							Host: "127.0.0.1",
+							Port: intstr.FromInt(12345),
+						},
+					}),
+				)
+			},
+		),
+		lc: &logging.Config{},
+		tc: &tracingconfig.Config{},
 		nc: &network.Config{
 			IstioOutboundIPRanges: "*",
 		},
 		oc: &metrics.ObservabilityConfig{},
-		ac: &autoscaler.Config{},
 		cc: &deployment.Config{},
 		want: makeDeployment(func(deploy *appsv1.Deployment) {
 			deploy.Spec.Template.ObjectMeta.Annotations[IstioOutboundIPRangeAnnotation] = "*"
@@ -774,11 +903,19 @@ func TestMakeDeployment(t *testing.T) {
 			revision.ObjectMeta.Annotations = map[string]string{
 				sidecarIstioInjectAnnotation: "false",
 			}
+			container(revision.Spec.GetContainer(),
+				withReadinessProbe(corev1.Handler{
+					TCPSocket: &corev1.TCPSocketAction{
+						Host: "127.0.0.1",
+						Port: intstr.FromInt(12345),
+					},
+				}),
+			)
 		}),
 		lc: &logging.Config{},
+		tc: &tracingconfig.Config{},
 		nc: &network.Config{},
 		oc: &metrics.ObservabilityConfig{},
-		ac: &autoscaler.Config{},
 		cc: &deployment.Config{},
 		want: makeDeployment(func(deploy *appsv1.Deployment) {
 			deploy.ObjectMeta.Annotations[sidecarIstioInjectAnnotation] = "false"
@@ -792,14 +929,22 @@ func TestMakeDeployment(t *testing.T) {
 				revision.ObjectMeta.Annotations = map[string]string{
 					IstioOutboundIPRangeAnnotation: "10.4.0.0/14,10.7.240.0/20",
 				}
+				container(revision.Spec.GetContainer(),
+					withReadinessProbe(corev1.Handler{
+						TCPSocket: &corev1.TCPSocketAction{
+							Host: "127.0.0.1",
+							Port: intstr.FromInt(12345),
+						},
+					}),
+				)
 			},
 		),
 		lc: &logging.Config{},
+		tc: &tracingconfig.Config{},
 		nc: &network.Config{
 			IstioOutboundIPRanges: "*",
 		},
 		oc: &metrics.ObservabilityConfig{},
-		ac: &autoscaler.Config{},
 		cc: &deployment.Config{},
 		want: makeDeployment(func(deploy *appsv1.Deployment) {
 			deploy.ObjectMeta.Annotations[IstioOutboundIPRangeAnnotation] = "10.4.0.0/14,10.7.240.0/20"
@@ -810,8 +955,15 @@ func TestMakeDeployment(t *testing.T) {
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
 			// Tested above so that we can rely on it here for brevity.
-			test.want.Spec.Template.Spec = *makePodSpec(test.rev, test.lc, test.oc, test.ac, test.cc)
-			got := MakeDeployment(test.rev, test.lc, test.nc, test.oc, test.ac, test.cc)
+			podSpec, err := makePodSpec(test.rev, test.lc, test.tc, test.oc, test.cc)
+			if err != nil {
+				t.Fatal("makePodSpec returned errror")
+			}
+			test.want.Spec.Template.Spec = *podSpec
+			got, err := MakeDeployment(test.rev, test.lc, test.tc, test.nc, test.oc, test.cc)
+			if err != nil {
+				t.Fatalf("got unexpected error: %v", err)
+			}
 			if diff := cmp.Diff(test.want, got, cmpopts.IgnoreUnexported(resource.Quantity{})); diff != "" {
 				t.Errorf("MakeDeployment (-want, +got) = %v", diff)
 			}
